@@ -22,6 +22,9 @@
 
 #include <netinet/in.h>		/* for IPPROTO_* */
 
+#include "shunk.h"
+#include "err.h"
+
 struct jambuf;
 
 /*
@@ -89,8 +92,13 @@ enum eroute_type {
 };
 #endif
 
-const struct ip_protocol *protocol_by_prefix(const char  *prefix);
+/* match then eat the start of prefix */
+const struct ip_protocol *protocol_by_caseeat_prefix(shunk_t *prefix);
+
 const struct ip_protocol *protocol_by_ipproto(unsigned protoid);
+const struct ip_protocol *protocol_by_shunk(shunk_t protocol);
+
+err_t ttoipproto(const char *text, unsigned *ipproto);
 
 /* ex: '=TCP=>' */
 size_t jam_protocols(struct jambuf *buf, const ip_protocol *src, char sep,
