@@ -236,12 +236,6 @@ extern void lsw_load_preshared_secrets(struct secret **psecrets, const char *sec
 				       struct logger *logger);
 extern void lsw_free_preshared_secrets(struct secret **psecrets, struct logger *logger);
 
-extern bool lsw_has_private_rawkey(const struct secret *secrets, const struct pubkey *pk);
-
-struct secret *find_secret_by_pubkey_ckaid(struct secret *secrets,
-					   const struct pubkey_type *type,
-					   const ckaid_t *pubkey_ckaid);
-
 extern struct secret *lsw_find_secret_by_id(struct secret *secrets,
 					    enum PrivateKeyKind kind,
 					    const struct id *my_id,
@@ -260,6 +254,7 @@ err_t find_or_load_private_key_by_ckaid(struct secret **secrets, const ckaid_t *
 
 /* these do not clone */
 chunk_t same_secitem_as_chunk(SECItem si);
+shunk_t same_secitem_as_shunk(SECItem si);
 SECItem same_chunk_as_secitem(chunk_t chunk, SECItemType type);
 
 chunk_t clone_secitem_as_chunk(SECItem si, const char *name);
