@@ -442,8 +442,8 @@ static const char *const exchange_name_ikev2[] = {
 };
 
 static const char *const exchange_name_private_use[] = {
-	"ISAKMP_XCHG_ECHOREQUEST_PRIVATE",	/* 244 - Used by libreswan  */
-	"ISAKMP_XCHG_ECHOREPLY_PRIVATE",	/* 245 - Used by libreswan  */
+	"ISAKMP_XCHG_ECHOREQUEST_PRIVATE",	/* 244 - Used by libreswan */
+	"ISAKMP_XCHG_ECHOREPLY_PRIVATE",	/* 245 - Used by libreswan */
 };
 
 static enum_names exchange_names_private_use = {
@@ -486,7 +486,7 @@ static enum_names exchange_names_doi_and_v2 = {
 	&ikev2_exchange_names
 };
 
-enum_names exchange_names_ikev1orv2 = {
+enum_names isakmp_xchg_type_names = {
 	ISAKMP_XCHG_NONE,
 	ISAKMP_XCHG_MODE_CFG,
 	ARRAY_REF(exchange_name_ikev1),
@@ -532,6 +532,7 @@ static const char *const sit_bit_name[] = {
 	P(SIT_IDENTITY_ONLY),
 	P(SIT_SECRECY),
 	P(SIT_INTEGRITY),
+#undef P
 };
 
 const struct enum_names sit_bit_names = {
@@ -599,9 +600,9 @@ static const char *const ah_transform_name[] = {
 	"AH_RIPEMD",
 	"AH_AES_XCBC_MAC",
 	"AH_RSA(UNUSED)",
-	"AH_AES_128_GMAC",	/* RFC4543 Errata1821  */
-	"AH_AES_192_GMAC",	/* RFC4543 Errata1821  */
-	"AH_AES_256_GMAC",	/* RFC4543 Errata1821  */
+	"AH_AES_128_GMAC",	/* RFC4543 Errata1821 */
+	"AH_AES_192_GMAC",	/* RFC4543 Errata1821 */
+	"AH_AES_256_GMAC",	/* RFC4543 Errata1821 */
 	/* 14-248 Unassigned */
 	/* 249-255 Reserved for private use */
 };
@@ -682,21 +683,24 @@ enum_names esp_transformid_names = {
 };
 
 /* IPCOMP transform values */
-static const char *const ipcomp_transform_name[] = {
-	"IPCOMP_OUI",
-	"IPCOMP_DEFLAT",
-	"IPCOMP_LZS",
-	"IPCOMP_LZJH",
+static const char *const ipsec_ipcomp_algo_name[] = {
+#define P(N) [N] = #N
+	P(IPCOMP_NONE),
+	P(IPCOMP_OUI),
+	P(IPCOMP_DEFLATE),
+	P(IPCOMP_LZS),
+	P(IPCOMP_LZJH),
 	/* 5-47 Reserved for approved algorithms */
 	/* 48-63 Reserved for private use */
 	/* 64-255 Unassigned */
+#undef P
 };
 
-enum_names ipcomp_transformid_names = {
-	IPCOMP_OUI,
+enum_names ipsec_ipcomp_algo_names = {
+	IPCOMP_NONE,
 	IPCOMP_LZJH,
-	ARRAY_REF(ipcomp_transform_name),
-	NULL, /* prefix */
+	ARRAY_REF(ipsec_ipcomp_algo_name),
+	"IPCOMP_", /* prefix */
 	NULL
 };
 
@@ -705,12 +709,12 @@ enum_names ipcomp_transformid_names = {
  * https://www.iana.org/assignments/ikev2-parameters/ikev2-parameters.xhtml#hash-algorithms
  */
 static const char *const ikev2_hash_algorithm_name[] = {
-        "IKEv2_HASH_ALGORITHM_RESERVED",
-        "IKEv2_HASH_ALGORITHM_SHA1",
-        "IKEv2_HASH_ALGORITHM_SHA2_256",
-        "IKEv2_HASH_ALGORITHM_SHA2_384",
-        "IKEv2_HASH_ALGORITHM_SHA2_512",
-        "IKEv2_HASH_ALGORITHM_IDENTITY"
+	"IKEv2_HASH_ALGORITHM_RESERVED",
+	"IKEv2_HASH_ALGORITHM_SHA1",
+	"IKEv2_HASH_ALGORITHM_SHA2_256",
+	"IKEv2_HASH_ALGORITHM_SHA2_384",
+	"IKEv2_HASH_ALGORITHM_SHA2_512",
+	"IKEv2_HASH_ALGORITHM_IDENTITY"
 	/* 6-1023 Unassigned */
 };
 
@@ -759,21 +763,23 @@ static const char *const ike_idtype_name[] = {
 	}
 
 /* IKEv1 */
-enum_names ike_idtype_names = ID_NR(ID_IPV4_ADDR, ID_NULL, NULL);
+enum_names ikev1_ike_id_type_names = ID_NR(ID_IPV4_ADDR, ID_NULL, NULL);
 
 /*
  * all names, including private-to-pluto
  * Tricky: lower bound and uppers bound are treated as unsigned long
  * so we have to tack two ranges onto ike_idtype_names.
+ *
+ * XXX: why not treat them as longs?
  */
-enum_names ike_idtype_names_extended0 = ID_NR(ID_NONE, ID_NONE, &ike_idtype_names);
-enum_names ike_idtype_names_extended = ID_NR(ID_FROMCERT, ID_FROMCERT, &ike_idtype_names_extended0);
+static enum_names ike_idtype_names_fromcert = ID_NR(ID_FROMCERT, ID_FROMCERT, NULL);
+enum_names ike_id_type_names = ID_NR(ID_NONE, ID_NULL, &ike_idtype_names_fromcert);
 
 /* IKEv2 names exclude ID_IPV4_ADDR_SUBNET, ID_IPV6_ADDR_SUBNET-ID_IPV6_ADDR_RANGE */
 
 static enum_names ikev2_idtype_names_3 = ID_NR(ID_DER_ASN1_DN, ID_NULL,	NULL);
 static enum_names ikev2_idtype_names_2 = ID_NR(ID_IPV6_ADDR, ID_IPV6_ADDR, &ikev2_idtype_names_3);
-enum_names ikev2_idtype_names = ID_NR(ID_IPV4_ADDR, ID_RFC822_ADDR, &ikev2_idtype_names_2);
+enum_names ikev2_ike_id_type_names = ID_NR(ID_IPV4_ADDR, ID_RFC822_ADDR, &ikev2_idtype_names_2);
 
 #undef ID_NR
 
@@ -1025,7 +1031,7 @@ enum_names *const ipsec_attr_val_descs[] = {
 	&sa_lifetime_names,	/* SA_LIFE_TYPE */
 	NULL,	/* SA_LIFE_DURATION */
 	&oakley_group_names,	/* GROUP_DESCRIPTION */
-	&enc_mode_names,	/* ENCAPSULATION_MODE */
+	&encapsulation_mode_names,
 	&auth_alg_names,	/* AUTH_ALGORITHM */
 	NULL,	/* KEY_LENGTH */
 	NULL,	/* KEY_ROUNDS */
@@ -1053,32 +1059,37 @@ enum_names sa_lifetime_names = {
 };
 
 /* Encapsulation Mode attribute */
-static const char *const enc_rfc_mode_name[] = {
-	"ENCAPSULATION_MODE_TUNNEL",
-	"ENCAPSULATION_MODE_TRANSPORT",
-	"ENCAPSULATION_MODE_UDP_TUNNEL_RFC",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT_RFC",
+
+static const char *const encapsulation_mode_draft_name[] = {
+#define P(N) [N - ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS] = #N
+	P(ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS),
+	P(ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS),
+#undef P
 };
 
-static const char *const enc_draft_mode_name[] = {
-	"ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS",
-};
-
-static enum_names enc_rfc_mode_names = {
-	ENCAPSULATION_MODE_TUNNEL,
-	ENCAPSULATION_MODE_UDP_TRANSPORT_RFC,
-	ARRAY_REF(enc_rfc_mode_name),
-	NULL, /* prefix */
-	NULL
-};
-
-enum_names enc_mode_names = {
+enum_names encapsulation_mode_draft_names = {
 	ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS,
 	ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS,
-	ARRAY_REF(enc_draft_mode_name),
-	NULL, /* prefix */
-	&enc_rfc_mode_names
+	ARRAY_REF(encapsulation_mode_draft_name),
+	"ENCAPSULATION_MODE_", /* prefix */
+	NULL,
+};
+
+static const char *const encapsulation_mode_rfc_name[] = {
+#define P(N) [N - ENCAPSULATION_MODE_TUNNEL] = #N
+	P(ENCAPSULATION_MODE_TUNNEL),
+	P(ENCAPSULATION_MODE_TRANSPORT),
+	P(ENCAPSULATION_MODE_UDP_TUNNEL_RFC),
+	P(ENCAPSULATION_MODE_UDP_TRANSPORT_RFC),
+#undef P
+};
+
+enum_names encapsulation_mode_names = {
+	ENCAPSULATION_MODE_TUNNEL,
+	ENCAPSULATION_MODE_UDP_TRANSPORT_RFC,
+	ARRAY_REF(encapsulation_mode_rfc_name),
+	"ENCAPSULATION_MODE_", /* prefix */
+	&encapsulation_mode_draft_names,
 };
 
 /* Auth Algorithm attribute */
@@ -1101,7 +1112,7 @@ static enum_names auth_alg_names_stolen_use = {
 	NULL
 };
 
-/* these string names map via a lookup function to configuration sttrings */
+/* these string names map via a lookup function to configuration strings */
 static const char *const auth_alg_name[] = {
 	"AUTH_ALGORITHM_NONE",	/* our own value, not standard */
 	"AUTH_ALGORITHM_HMAC_MD5",
@@ -1149,7 +1160,7 @@ enum_names xauth_type_names = {
 	NULL
 };
 
-/* IKEv1 XAUTH-STATUS attribute names  */
+/* IKEv1 XAUTH-STATUS attribute names */
 static const char *const modecfg_attr_name_draft[] = {
 	"INTERNAL_IP4_ADDRESS",	/* 1 */
 	"INTERNAL_IP4_NETMASK",
@@ -1352,7 +1363,7 @@ enum_names oakley_enc_names = {
  * https://www.iana.org/assignments/ipsec-registry/ipsec-registry.xhtml#ipsec-registry-6
  */
 
-/* these string names map via a lookup function to configuration sttrings */
+/* these string names map via a lookup function to configuration strings */
 static const char *const oakley_hash_name[] = {
 	/* 0 - RESERVED */
 	"OAKLEY_MD5",
@@ -1522,7 +1533,7 @@ enum_names ikev2_auth_names = {
  * be differences we need to care about)
  */
 
-/* these string names map via a lookup function to configuration sttrings */
+/* these string names map via a lookup function to configuration strings */
 static const char *const oakley_group_name[] = {
 	"OAKLEY_GROUP_NONE", /* 0! RFC 7296 */
 	"OAKLEY_GROUP_MODP768",
@@ -2203,22 +2214,132 @@ enum_names ikev2_redirect_gw_names = {
 	NULL
 };
 
-/* magic SPI values (specific to Libreswan: see <libreswan.h>) */
-static const char *const spi_name[] = {
+/* magic SPI values (specific to Libreswan */
+
+static const char *const policy_spi_name[] = {
 	"%pass",
 	"%drop",
 	"%reject",
 	"%hold",
 	"%trap",
+	"%ignore",
 	"%trapsubnet",
 };
-enum_names spi_names = {
+
+enum_names policy_spi_names = {
 	SPI_PASS, SPI_TRAPSUBNET,
-	ARRAY_REF(spi_name),
+	ARRAY_REF(policy_spi_name),
 	"%",	/* prefix */
 	NULL
 };
 
+/*
+ * enum global_timers
+ */
+
+static const char *global_timer_name[] = {
+#define E(T) [T] = #T
+	E(EVENT_REINIT_SECRET),
+	E(EVENT_SHUNT_SCAN),
+	E(EVENT_PENDING_DDNS),
+	E(EVENT_SD_WATCHDOG),
+	E(EVENT_PENDING_PHASE2),
+	E(EVENT_CHECK_CRLS),
+	E(EVENT_REVIVE_CONNS),
+	E(EVENT_FREE_ROOT_CERTS),
+	E(EVENT_RESET_LOG_RATE_LIMIT),
+	E(EVENT_PROCESS_KERNEL_QUEUE),
+	E(EVENT_NAT_T_KEEPALIVE),
+#undef E
+};
+const struct enum_names global_timer_names = {
+	0, elemsof(global_timer_name) - 1,
+	ARRAY_REF(global_timer_name),
+	"EVENT_",
+	NULL,
+};
+
+/*
+ * enum event_type
+ */
+
+static const char *const event_name[] = {
+#define E(EVENT) [EVENT - EVENT_NULL] = #EVENT
+	E(EVENT_NULL),
+	E(EVENT_RETRANSMIT),
+	E(EVENT_DPD),
+	E(EVENT_DPD_TIMEOUT),
+	E(EVENT_CRYPTO_TIMEOUT),
+	E(EVENT_PAM_TIMEOUT),
+#undef E
+};
+
+static const enum_names event_names = {
+	EVENT_NULL, EVENT_PAM_TIMEOUT,
+	ARRAY_REF(event_name),
+	"EVENT_", /* prefix */
+	NULL
+};
+
+static const char *const event_sa_name[] = {
+#define E(EVENT) [EVENT - EVENT_SA_DISCARD] = #EVENT
+	E(EVENT_SA_DISCARD),
+	E(EVENT_SA_REPLACE),
+	E(EVENT_SA_EXPIRE),
+#undef E
+};
+
+static const enum_names event_sa_names = {
+	EVENT_SA_DISCARD, EVENT_SA_EXPIRE,
+	ARRAY_REF(event_sa_name),
+	"EVENT_SA_", /* prefix */
+	&event_names,
+};
+
+static const char *const event_v1_name[] = {
+#define E(EVENT) [EVENT - EVENT_v1_SEND_XAUTH] = #EVENT
+	E(EVENT_v1_SEND_XAUTH),
+	E(EVENT_v1_REPLACE_IF_USED),
+#undef E
+};
+
+static const enum_names event_v1_names = {
+	EVENT_v1_SEND_XAUTH, EVENT_v1_REPLACE_IF_USED,
+	ARRAY_REF(event_v1_name),
+	"EVENT_v1_", /* prefix */
+	&event_sa_names
+};
+
+static const char *const event_v2_name[] = {
+#define E(EVENT) [EVENT - EVENT_v2_REKEY] = #EVENT
+	E(EVENT_v2_REKEY),
+	E(EVENT_v2_REAUTH),
+	E(EVENT_v2_LIVENESS),
+	E(EVENT_v2_ADDR_CHANGE),
+	E(EVENT_v2_REDIRECT),
+#undef E
+};
+
+static const enum_names event_v2_names = {
+	EVENT_v2_REKEY, EVENT_v2_REDIRECT,
+	ARRAY_REF(event_v2_name),
+	"EVENT_v2_", /* prefix */
+	&event_v1_names,
+};
+
+static const char *const event_retain_name[] = {
+#define E(EVENT) [EVENT - EVENT_RETAIN] = #EVENT
+	E(EVENT_RETAIN),
+#undef E
+};
+
+const enum_names event_type_names = {
+	EVENT_RETAIN, EVENT_RETAIN,
+	ARRAY_REF(event_retain_name),
+	"EVENT_", /* prefix */
+	&event_v2_names,
+};
+
 /*
  * Iterate over the enum_names returning all the valid indexes.
  *
@@ -2315,7 +2436,7 @@ const char *enum_name(enum_names *ed, unsigned long val)
 	return enum_range_name(range, val, prefix, /*shorten?*/false);
 }
 
-const char *enum_short_name(enum_names *ed, unsigned long val)
+const char *enum_name_short(enum_names *ed, unsigned long val)
 {
 	const char *prefix = NULL;
 	/* can be NULL */
@@ -2339,7 +2460,7 @@ size_t jam_enum(struct jambuf *buf, enum_names *en, unsigned long val)
 
 size_t jam_enum_short(struct jambuf *buf, enum_names *en, unsigned long val)
 {
-	const char *name = enum_short_name(en, val);
+	const char *name = enum_name_short(en, val);
 	if (name == NULL) {
 		if (en->en_prefix != NULL) {
 			jam_string(buf, en->en_prefix);
@@ -2355,7 +2476,7 @@ size_t jam_enum_short(struct jambuf *buf, enum_names *en, unsigned long val)
  *
  * Note: result may or may not be in b.
  */
-const char *enum_showb(enum_names *ed, unsigned long val, struct esb_buf *b)
+const char *enum_show(enum_names *ed, unsigned long val, esb_buf *b)
 {
 	const char *p = enum_name(ed, val);
 
@@ -2366,7 +2487,7 @@ const char *enum_showb(enum_names *ed, unsigned long val, struct esb_buf *b)
 	return p;
 }
 
-const char *enum_show_shortb(enum_names *ed, unsigned long val, struct esb_buf *b)
+const char *enum_show_short(enum_names *ed, unsigned long val, esb_buf *b)
 {
 	const char *prefix;
 	const struct enum_names *range = enum_range(ed, val, &prefix);
@@ -2379,22 +2500,6 @@ const char *enum_show_shortb(enum_names *ed, unsigned long val, struct esb_buf *
 	return name;
 }
 
-/*
- * find or construct a string to describe an enum value
- * Result may be in STATIC buffer -- NOT RE-ENTRANT!
- *
- * One consequence is that you cannot have two or more calls
- * as arguments in a single logging call.  Use enum_name instead.
- * (Of course that means that unnamed values will be shown
- * badly.)
- */
-const char *enum_show(enum_names *ed, unsigned long val)
-{
-	static struct esb_buf buf;	/* only one! NON-RE-ENTRANT */
-
-	return enum_showb(ed, val, &buf);
-}
-
 /*
  * Find the value for a name in an enum_names table.  If not found, returns -1.
  *
@@ -2456,7 +2561,7 @@ int enum_match(enum_names *ed, shunk_t string)
 			if (try(true, 0, 0) ||
 			    try(suffix_len > 0, 0, suffix_len) ||
 			    try(prefix_len > 0, prefix_len, 0) ||
-			    try(suffix_len > 0 && suffix_len > 0, prefix_len, suffix_len))
+			    try(prefix_len > 0 && suffix_len > 0, prefix_len, suffix_len))
 			{
 				return en;
 			}
@@ -2487,8 +2592,8 @@ const char *enum_enum_name(enum_enum_names *een, unsigned long table,
 	return en == NULL ? NULL : enum_name(en, val);
 }
 
-const char *enum_enum_showb(enum_enum_names *een, unsigned long table,
-			    unsigned long val, struct esb_buf *b)
+const char *enum_enum_show(enum_enum_names *een, unsigned long table,
+			   unsigned long val, esb_buf *b)
 {
 	enum_names *en = enum_enum_table(een, table);
 	if (en == NULL) {
@@ -2497,11 +2602,11 @@ const char *enum_enum_showb(enum_enum_names *een, unsigned long table,
 		return b->buf;
 	}
 
-	return enum_showb(en, val, b);
+	return enum_show(en, val, b);
 }
 
-const char *enum_enum_show_shortb(enum_enum_names *een, unsigned long table,
-				  unsigned long val, struct esb_buf *b)
+const char *enum_enum_show_short(enum_enum_names *een, unsigned long table,
+				 unsigned long val, esb_buf *b)
 {
 	enum_names *en = enum_enum_table(een, table);
 	if (en == NULL) {
@@ -2510,7 +2615,7 @@ const char *enum_enum_show_shortb(enum_enum_names *een, unsigned long table,
 		return b->buf;
 	}
 
-	return enum_show_shortb(en, val, b);
+	return enum_show_short(en, val, b);
 }
 
 size_t jam_enum_enum(struct jambuf *buf, enum_enum_names *een,
@@ -2559,7 +2664,7 @@ const char *sparse_val_show(sparse_names sd, unsigned long val)
 	const char *p = sparse_name(sd, val);
 
 	if (p == NULL) {
-		static struct esb_buf b;	/* STATIC!! */
+		static esb_buf b;	/* STATIC!! */
 
 		snprintf(b.buf, sizeof(b.buf), "%lu??", val);
 		p = b.buf;
@@ -2577,21 +2682,21 @@ static const enum_names *en_checklist[] = {
 	&payload_names_ikev1orv2,
 	&ikev2_last_proposal_desc,
 	&ikev1_exchange_names,
-	&exchange_names_ikev1orv2,
+	&isakmp_xchg_type_names,
 	&ikev1_protocol_names,
 	&isakmp_transformid_names,
 	&ah_transformid_names,
 	&esp_transformid_names,
-	&ipcomp_transformid_names,
-	&ike_idtype_names,
-	&ikev2_idtype_names,
+	&ipsec_ipcomp_algo_names,
+	&ikev1_ike_id_type_names,
+	&ikev2_ike_id_type_names,
 	&ike_cert_type_names,
 	&ikev2_cert_type_names,
 	&certpolicy_type_names,
 	&oakley_attr_names,
 	&ipsec_attr_names,
 	&sa_lifetime_names,
-	&enc_mode_names,
+	&encapsulation_mode_names,
 	&auth_alg_names,
 	&xauth_type_names,
 	&modecfg_attr_names,
@@ -2628,6 +2733,9 @@ static const enum_names *en_checklist[] = {
 	&ike_version_child_names,
 	&payload_flag_names,
 	&oakley_attr_bit_names,
+	&global_timer_names,
+	&event_type_names,
+	&policy_spi_names,
 };
 
 void check_enum_names(enum_names *checklist[], size_t tl)
