@@ -69,10 +69,17 @@ static char *environlize(const char *str)
 static void resolve_defaultroute(struct starter_conn *conn UNUSED, struct logger *logger)
 {
 #ifdef XFRM_SUPPORT
-	if (resolve_defaultroute_one(&conn->left, &conn->right, verbose != 0, logger) == 1)
-		resolve_defaultroute_one(&conn->left, &conn->right, verbose != 0, logger);
-	if (resolve_defaultroute_one(&conn->right, &conn->left, verbose != 0, logger) == 1)
-		resolve_defaultroute_one(&conn->right, &conn->left, verbose != 0, logger);
+	lset_t verbose_rc_flags = verbose ? (WHACK_STREAM|NO_PREFIX) : LEMPTY;
+	if (resolve_defaultroute_one(&conn->left, &conn->right, verbose_rc_flags,
+				     logger) == RESOLVE_PLEASE_CALL_AGAIN) {
+		resolve_defaultroute_one(&conn->left, &conn->right,
+					 verbose_rc_flags, logger);
+	}
+	if (resolve_defaultroute_one(&conn->right, &conn->left, verbose_rc_flags,
+				     logger) == RESOLVE_PLEASE_CALL_AGAIN) {
+		resolve_defaultroute_one(&conn->right, &conn->left,
+					 verbose_rc_flags, logger);
+	}
 #else /* !defined(XFRM_SUPPORT) */
 	/* What kind of result are we seeking? */
 	bool seeking_src = (conn->left.addrtype == KH_DEFAULTROUTE ||
@@ -83,7 +90,7 @@ static void resolve_defaultroute(struct starter_conn *conn UNUSED, struct logger
 		return;	/* this end already figured out */
 
 	fatal(PLUTO_EXIT_FAIL, logger,
-	      "addcon: without XFRM/NETKEY, cannot resolve_defaultroute()");
+	      "addcon: without XFRM, cannot resolve_defaultroute()");
 #endif
 }
 
@@ -218,18 +225,18 @@ int main(int argc, char *argv[])
 	struct logger *logger = tool_init_log(argv[0]);
 
 	int opt;
-	bool autoall = FALSE;
-	bool configsetup = FALSE;
-	bool checkconfig = FALSE;
+	bool autoall = false;
+	bool configsetup = false;
+	bool checkconfig = false;
 	const char *export = "export"; /* display export before the foo=bar or not */
 	bool
-		dolist = FALSE,
-		listadd = FALSE,
-		listroute = FALSE,
-		liststart = FALSE,
-		listignore = FALSE,
-		listall = FALSE,
-		liststack = FALSE;
+		dolist = false,
+		listadd = false,
+		listroute = false,
+		liststart = false,
+		listignore = false,
+		listall = false,
+		liststack = false;
 	char *configfile = NULL;
 	const char *varprefix = "";
 	int exit_status = 0;
@@ -252,7 +259,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'a':
-			autoall = TRUE;
+			autoall = true;
 			break;
 
 		case 'D':
@@ -261,11 +268,11 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'T':
-			configsetup = TRUE;
+			configsetup = true;
 			break;
 
 		case 'K':
-			checkconfig = TRUE;
+			checkconfig = true;
 			break;
 
 		case 'N':
@@ -281,33 +288,33 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'L':
-			listadd = TRUE;
-			dolist = TRUE;
+			listadd = true;
+			dolist = true;
 			break;
 
 		case 'r':
-			listroute = TRUE;
-			dolist = TRUE;
+			listroute = true;
+			dolist = true;
 			break;
 
 		case 's':
-			liststart = TRUE;
-			dolist = TRUE;
+			liststart = true;
+			dolist = true;
 			break;
 
 		case 'S':
-			liststack = TRUE;
-			dolist = TRUE;
+			liststack = true;
+			dolist = true;
 			break;
 
 		case 'i':
-			listignore = TRUE;
-			dolist = TRUE;
+			listignore = true;
+			dolist = true;
 			break;
 
 		case 'A':
-			listall = TRUE;
-			dolist = TRUE;
+			listall = true;
+			dolist = true;
 			break;
 
 		case 'P':
@@ -364,7 +371,7 @@ int main(int argc, char *argv[])
 	if (verbose > 0)
 		printf("opening file: %s\n", configfile);
 
-	starter_use_log(verbose != 0, TRUE, verbose == 0);
+	starter_use_log(verbose != 0, true, verbose == 0);
 
 	struct starter_config *cfg = NULL;
 
@@ -380,7 +387,7 @@ int main(int argc, char *argv[])
 			exit(3);
 		}
 		if (errl.errors != NULL) {
-			fprintf(stderr, "addconn, in config '%s', ignoring: %s\n",
+			fprintf(stderr, "addconn, in config '%s', %s\n",
 				configfile, errl.errors);
 			pfree(errl.errors);
 		}
@@ -433,7 +440,7 @@ int main(int argc, char *argv[])
 				conn->desired_state == STARTUP_START)
 			{
 				if (verbose > 0)
-					printf(" %s", conn->name);
+					printf(" %s\n", conn->name);
 				resolve_defaultroute(conn, logger);
 				starter_whack_add_conn(cfg, conn, logger);
 			}
@@ -485,7 +492,7 @@ int main(int argc, char *argv[])
 			const char *p3 = "";
 
 			if (verbose > 0)
-				printf(" %s", connname);
+				printf(" %s\n", connname);
 
 			/* find first name match, if any */
 			for (conn = cfg->conns.tqh_first;
@@ -614,7 +621,7 @@ int main(int argc, char *argv[])
 						cfg->setup.strings[kd->field]);
 				} else {
 					/* implicit default */
-					printf("netkey\n");
+					printf("xfrm\n");
 				}
 			}
 		}
