@@ -50,7 +50,6 @@
 #include "kernel.h"     /* needs connections.h */
 #include "log.h"
 #include "server.h"
-#include "spdb.h"
 #include "timer.h"
 #include "rnd.h"
 #include "ipsec_doi.h"  /* needs demux.h and state.h */
@@ -63,6 +62,71 @@
 
 #include "pluto_stats.h"
 
+static void dpd_clear_connection(struct connection *c, const char *v)
+{
+	/*
+	 * For CK_INSTANCE, delete_states_by_connection() will clear
+	 * Note that delete_states_by_connection changes c->kind but we need
+	 * to remember what it was to know if we still need to unroute after delete
+	 */
+	if (c->kind == CK_INSTANCE) {
+		delete_states_by_connection(c, /*relations?*/true);
+	} else {
+		flush_pending_by_connection(c); /* remove any partial negotiations that are failing */
+		delete_states_by_connection(c, /*relations?*/true);
+		dbg("%s: unrouting connection %s action - clearing",
+		    enum_name(&connection_kind_names, c->kind), v);
+		unroute_connection(c); /* --unroute */
+	}
+}
+
+static void dpd_action(struct state *tbd_st)
+{
+	/*
+	 * XXX: Danger! These connection calls, notably
+	 * restart_connections_by_peer(), can end up deleting TBD_ST
+	 *
+	 * So that the logger is valid after TBD_ST's been deleted,
+	 * create a clone of TBD_ST's logger and kill the TBD_ST
+	 * pointer.
+	 */
+	struct logger *logger = clone_logger(tbd_st->st_logger, HERE);
+	struct connection *c = tbd_st->st_connection;
+	const char *liveness_name = enum_name(&ike_version_liveness_names, tbd_st->st_ike_version);
+	passert(liveness_name != NULL);
+	tbd_st = NULL; /* kill TBD_ST; can no longer be trusted */
+
+	switch (c->dpd_action) {
+	case DPD_ACTION_CLEAR:
+		llog(RC_LOG, logger,
+		     "%s action - clearing connection kind %s",
+		     liveness_name, enum_name(&connection_kind_names, c->kind));
+		dpd_clear_connection(c, liveness_name);
+		break;
+
+	case DPD_ACTION_RESTART:
+		llog(RC_LOG, logger,
+		     "%s action - restarting all connections that share this peer",
+		     liveness_name);
+		restart_connections_by_peer(c, logger);
+		break;
+
+	case DPD_ACTION_HOLD:
+		llog(RC_LOG, logger,
+		     "%s action - putting connection into hold",
+		     liveness_name);
+		if (c->kind == CK_INSTANCE) {
+			dbg("%s warning dpdaction=hold on instance futile - will be deleted", liveness_name);
+		}
+		delete_states_by_connection(c, /*relations?*/true);
+		break;
+
+	default:
+		bad_case(c->dpd_action);
+	}
+	free_logger(&logger, HERE);
+}
+
 /*
  * Initialize RFC 3706 Dead Peer Detection
  *
@@ -122,40 +186,41 @@ stf_status dpd_init(struct state *st)
 	bool want_dpd = dpd_active_locally(st);
 
 	if (IS_IKE_SA(st)) { /* so we log this only once */
-		DBG(DBG_DPD, DBG_log("DPD: dpd_init() called on ISAKMP SA"));
+		dbg("DPD: dpd_init() called on ISAKMP SA");
 
 		if (!peer_supports_dpd) {
-			DBG(DBG_DPD, DBG_log("DPD: Peer does not support Dead Peer Detection"));
+			dbg("DPD: Peer does not support Dead Peer Detection");
 			if (want_dpd)
-				loglog(RC_LOG_SERIOUS,
-					"Configured DPD (RFC 3706) support not enabled because remote peer did not advertise DPD support");
+				log_state(RC_LOG_SERIOUS, st,
+					  "Configured DPD (RFC 3706) support not enabled because remote peer did not advertise DPD support");
 			return STF_OK;
 		} else {
-			DBG(DBG_DPD, DBG_log("DPD: Peer supports Dead Peer Detection"));
+			dbg("DPD: Peer supports Dead Peer Detection");
 		}
 
 		if (!want_dpd) {
-			DBG(DBG_DPD, DBG_log("DPD: not initializing DPD because DPD is disabled locally"));
+			dbg("DPD: not initializing DPD because DPD is disabled locally");
 			return STF_OK;
 		}
 	} else {
-		DBG(DBG_DPD, DBG_log("DPD: dpd_init() called on IPsec SA"));
+		dbg("DPD: dpd_init() called on IPsec SA");
 		if (!peer_supports_dpd || !want_dpd) {
-			DBG(DBG_DPD, DBG_log("DPD: Peer does not support Dead Peer Detection"));
+			dbg("DPD: Peer does not support Dead Peer Detection");
 			return STF_OK;
 		}
 
 		/* find the IKE SA */
 		struct state *p1st = find_state_ikev1(&st->st_ike_spis, 0);
 		if (p1st == NULL) {
-			loglog(RC_LOG_SERIOUS, "could not find phase 1 state for DPD");
+			log_state(RC_LOG_SERIOUS, st,
+				  "could not find phase 1 state for DPD");
 			return STF_FAIL;
 		}
 
 		if (st->st_dpd_event == NULL || ev_before(st->st_dpd_event,
 			st->st_connection->dpd_delay))
 		{
-			delete_dpd_event(st);
+			event_delete(EVENT_DPD, st);
 			event_schedule(EVENT_DPD, st->st_connection->dpd_delay, st);
 		}
 	}
@@ -170,10 +235,9 @@ static void dpd_sched_timeout(struct state *p1st, monotime_t nw, deltatime_t tim
 {
 	passert(deltasecs(timeout) > 0);
 	if (p1st->st_dpd_event == NULL ||
-	    monobefore(monotimesum(nw, timeout), p1st->st_dpd_event->ev_time)) {
-		DBG(DBG_DPD, DBG_log("DPD: scheduling timeout to %ld",
-				     (long)deltasecs(timeout)));
-		delete_dpd_event(p1st);
+	    monobefore(monotime_add(nw, timeout), p1st->st_dpd_event->ev_time)) {
+		dbg("DPD: scheduling timeout to %jd", deltasecs(timeout));
+		event_delete(EVENT_DPD, p1st);
 		event_schedule(EVENT_DPD_TIMEOUT, timeout, p1st);
 	}
 }
@@ -189,25 +253,21 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 {
 	uint32_t seqno;
 
-	DBG(DBG_DPD, {
-		char cib[CONN_INST_BUF];
-		DBG_log("DPD: processing for state #%lu (\"%s\"%s)",
-			st->st_serialno,
-			st->st_connection->name,
-			fmt_conn_instance(st->st_connection, cib));
-	});
+	connection_buf cib;
+	dbg("DPD: processing for state #%lu ("PRI_CONNECTION")",
+	    st->st_serialno, pri_connection(st->st_connection, &cib));
 
 	/* if peer doesn't support DPD, DPD should never have started */
 	pexpect(st->hidden_variables.st_peer_supports_dpd);	/* ??? passert? */
 	if (!st->hidden_variables.st_peer_supports_dpd) {
-		DBG(DBG_DPD, DBG_log("DPD: peer does not support dpd"));
+		dbg("DPD: peer does not support dpd");
 		return;
 	}
 
 	/* If there is no IKE state, there can be no DPD */
-	pexpect(IS_ISAKMP_SA_ESTABLISHED(p1st->st_state));	/* ??? passert? */
-	if (!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state)) {
-		DBG(DBG_DPD, DBG_log("DPD: no phase1 state, so no DPD"));
+	pexpect(IS_V1_ISAKMP_SA_ESTABLISHED(p1st));	/* ??? passert? */
+	if (!IS_V1_ISAKMP_SA_ESTABLISHED(p1st)) {
+		dbg("DPD: no phase1 state, so no DPD");
 		return;
 	}
 
@@ -229,18 +289,16 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 	monotime_t last = !monobefore(p1st->st_last_dpd, st->st_last_dpd) ?
 		p1st->st_last_dpd : st->st_last_dpd;
 
-	monotime_t next_time = monotimesum(last, delay);
+	monotime_t next_time = monotime_add(last, delay);
 	deltatime_t next_delay = monotimediff(next_time, nw);
 
 	/* has there been enough activity of late? */
-	if (deltatime_cmp(next_delay, deltatime(0)) > 0) {
+	if (deltatime_cmp(next_delay, >, deltatime(0))) {
 		/* Yes, just reschedule "phase 2" */
-		LSWDBGP(DBG_DPD, buf) {
-			lswlogs(buf, "DPD: not yet time for dpd event: ");
-			lswlog_monotime(buf, nw);
-			lswlogs(buf, " < ");
-			lswlog_monotime(buf, next_time);
-		}
+		monotime_buf mb1, mb2;
+		dbg("DPD: not yet time for dpd event: %s < %s",
+		    str_monotime(nw, &mb1),
+		    str_monotime(next_time, &mb2));
 		event_schedule(EVENT_DPD, next_delay, st);
 		return;
 	}
@@ -254,7 +312,7 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 	if (eroute_care && st->hidden_variables.st_nat_traversal == LEMPTY &&
 			!was_eroute_idle(st, delay))
 	{
-		DBG(DBG_DPD, DBG_log("DPD: out event not sent, phase 2 active"));
+		dbg("DPD: out event not sent, phase 2 active");
 
 		/* update phase 2 time stamp only */
 		st->st_last_dpd = nw;
@@ -267,9 +325,8 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 		 */
 		if (p1st->st_dpd_event != NULL &&
 		    p1st->st_dpd_event->ev_type == EVENT_DPD_TIMEOUT) {
-			DBG(DBG_DPD,
-			    DBG_log("DPD: deleting p1st DPD event"));
-			delete_dpd_event(p1st);
+			dbg("DPD: deleting p1st DPD event");
+			event_delete(EVENT_DPD, p1st);
 		}
 
 		event_schedule(EVENT_DPD, next_delay, st);
@@ -286,7 +343,7 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 
 	if (p1st->st_dpd_seqno == 0) {
 		/* Get a non-zero random value that has room to grow */
-		get_rnd_bytes((u_char *)&p1st->st_dpd_seqno,
+		get_rnd_bytes((uint8_t *)&p1st->st_dpd_seqno,
 			      sizeof(p1st->st_dpd_seqno));
 		p1st->st_dpd_seqno &= 0x7fff;
 		p1st->st_dpd_seqno++;
@@ -307,7 +364,8 @@ static void dpd_outI(struct state *p1st, struct state *st, bool eroute_care,
 
 	if (send_isakmp_notification(p1st, R_U_THERE,
 				     &seqno, sizeof(seqno)) != STF_IGNORE) {
-		loglog(RC_LOG_SERIOUS, "DPD: could not send R_U_THERE");
+		log_state(RC_LOG_SERIOUS, st,
+			  "DPD: could not send R_U_THERE");
 		return;
 	}
 
@@ -322,7 +380,7 @@ static void p1_dpd_outI1(struct state *p1st)
 	deltatime_t delay = p1st->st_connection->dpd_delay;
 	deltatime_t timeout = p1st->st_connection->dpd_timeout;
 
-	dpd_outI(p1st, p1st, TRUE, delay, timeout);
+	dpd_outI(p1st, p1st, true, delay, timeout);
 }
 
 static void p2_dpd_outI1(struct state *p2st)
@@ -332,31 +390,29 @@ static void p2_dpd_outI1(struct state *p2st)
 	deltatime_t timeout = p2st->st_connection->dpd_timeout;
 
 	st = find_phase1_state(p2st->st_connection,
-		ISAKMP_SA_ESTABLISHED_STATES);
+		V1_ISAKMP_SA_ESTABLISHED_STATES);
 
 	if (st == NULL) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: could not find newest phase 1 state - initiating a new one");
-		liveness_action(p2st->st_connection, p2st->st_ike_version);
+		log_state(RC_LOG_SERIOUS, p2st,
+			  "DPD: could not find newest phase 1 state - initiating a new one");
+		dpd_action(p2st);
 		return;
 	}
 
 	if (st->st_connection->newest_ipsec_sa != p2st->st_serialno) {
-		DBG(DBG_DPD,
-		    DBG_log("DPD: no need to send or schedule DPD for replaced IPsec SA"));
+		dbg("DPD: no need to send or schedule DPD for replaced IPsec SA");
 		return;
 	}
 
-	dpd_outI(st, p2st, TRUE, delay, timeout);
+	dpd_outI(st, p2st, true, delay, timeout);
 }
 
 void dpd_event(struct state *st)
 {
 	passert(st != NULL);
 
-	set_cur_state(st);
 
-	if (IS_PHASE1(st->st_state->kind) || IS_PHASE15(st->st_state->kind))
+	if (IS_V1_PHASE1(st->st_state->kind) || IS_V1_PHASE15(st->st_state->kind))
 		p1_dpd_outI1(st);
 	else
 		p2_dpd_outI1(st);
@@ -377,48 +433,46 @@ stf_status dpd_inI_outR(struct state *p1st,
 	monotime_t nw = mononow();
 	uint32_t seqno;
 
-	if (!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state)) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: received R_U_THERE for unestablished ISKAMP SA");
+	if (!IS_V1_ISAKMP_SA_ESTABLISHED(p1st)) {
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: received R_U_THERE for unestablished ISKAMP SA");
 		return STF_IGNORE;
 	}
 	if (n->isan_spisize != COOKIE_SIZE * 2 ||
 	    pbs_left(pbs) < COOKIE_SIZE * 2) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: R_U_THERE has invalid SPI length (%d)",
-		       n->isan_spisize);
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: R_U_THERE has invalid SPI length (%d)",
+			  n->isan_spisize);
 		return STF_FAIL + PAYLOAD_MALFORMED;
 	}
 
 	if (!memeq(pbs->cur, p1st->st_ike_spis.initiator.bytes, COOKIE_SIZE)) {
 		/* RFC states we *SHOULD* check cookies, not MUST.  So invalid
 		   cookies are technically valid, as per Geoffrey Huang */
-		DBG(DBG_DPD,
-		    DBG_log("DPD: R_U_THERE has invalid icookie (tolerated)"));
+		dbg("DPD: R_U_THERE has invalid icookie (tolerated)");
 	}
 	pbs->cur += COOKIE_SIZE;
 
 	if (!memeq(pbs->cur, p1st->st_ike_spis.responder.bytes, COOKIE_SIZE)) {
-		DBG(DBG_DPD,
-		    DBG_log("DPD: R_U_THERE has invalid rcookie (tolerated)"));
+		dbg("DPD: R_U_THERE has invalid rcookie (tolerated)");
 	}
 	pbs->cur += COOKIE_SIZE;
 
 	if (pbs_left(pbs) != sizeof(seqno)) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: R_U_THERE has invalid data length (%d)", (int) pbs_left(
-			       pbs));
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: R_U_THERE has invalid data length (%d)",
+			  (int) pbs_left(pbs));
 		return STF_FAIL + PAYLOAD_MALFORMED;
 	}
 
 	seqno = ntohl(*(uint32_t *)pbs->cur);
 	if (p1st->st_dpd_peerseqno && seqno <= p1st->st_dpd_peerseqno) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: received old or duplicate R_U_THERE");
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: received old or duplicate R_U_THERE");
 		if (p1st->st_dpd_rdupcount >= DPD_RETRANS_MAX) {
-			loglog(RC_LOG_SERIOUS,
-		       "DPD: received %d or more duplicate R_U_THERE's - will no longer answer",
-				DPD_RETRANS_MAX);
+			log_state(RC_LOG_SERIOUS, p1st,
+				  "DPD: received %d or more duplicate R_U_THERE's - will no longer answer",
+				  DPD_RETRANS_MAX);
 			return STF_IGNORE;
 		} else {
 			/*
@@ -427,31 +481,28 @@ stf_status dpd_inI_outR(struct state *p1st,
 			 * isakmp_sa->dpd_seq on unanswered DPD probe violating
 			 * RFC 3706 Section 7 "Security Considerations"
 			 */
-			loglog(RC_LOG_SERIOUS,
-		       "DPD: received less than %d duplicate R_U_THERE's - will reluctantly answer",
-				DPD_RETRANS_MAX);
+			log_state(RC_LOG_SERIOUS, p1st,
+				  "DPD: received less than %d duplicate R_U_THERE's - will reluctantly answer",
+				  DPD_RETRANS_MAX);
 			p1st->st_dpd_rdupcount++;
 		}
 	} else {
 		p1st->st_dpd_rdupcount = 0;
 	}
 
-	LSWDBGP(DBG_DPD, buf) {
-		lswlogf(buf, "DPD: received R_U_THERE seq:%u monotime:",
-			seqno);
-		lswlog_monotime(buf, nw);
-		char cib[CONN_INST_BUF];
-		lswlogf(buf, " (state=#%lu name=\"%s\"%s)",
-			p1st->st_serialno,
-			p1st->st_connection->name,
-			fmt_conn_instance(p1st->st_connection, cib));
-	};
+	monotime_buf nwb;
+	connection_buf cib;
+	dbg("DPD: received R_U_THERE seq:%u monotime: %s (state=#%lu name="PRI_CONNECTION")",
+	    seqno, str_monotime(nw, &nwb),
+	    p1st->st_serialno,
+	    pri_connection(p1st->st_connection, &cib));
 
 	p1st->st_dpd_peerseqno = seqno;
 
 	if (send_isakmp_notification(p1st, R_U_THERE_ACK,
 				     pbs->cur, pbs_left(pbs)) != STF_IGNORE) {
-		loglog(RC_LOG_SERIOUS, "DPD: could not send R_U_THERE_ACK");
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: could not send R_U_THERE_ACK");
 		return STF_IGNORE;
 	}
 
@@ -466,7 +517,7 @@ stf_status dpd_inI_outR(struct state *p1st,
 	 */
 	if (p1st->st_dpd_event != NULL &&
 	    p1st->st_dpd_event->ev_type == EVENT_DPD_TIMEOUT)
-		delete_dpd_event(p1st);
+		event_delete(EVENT_DPD, p1st);
 
 	return STF_IGNORE;
 }
@@ -485,56 +536,53 @@ stf_status dpd_inR(struct state *p1st,
 {
 	uint32_t seqno;
 
-	if (!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state)) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: received R_U_THERE_ACK for unestablished ISKAMP SA");
+	if (!IS_V1_ISAKMP_SA_ESTABLISHED(p1st)) {
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: received R_U_THERE_ACK for unestablished ISKAMP SA");
 		return STF_FAIL;
 	}
 
 	if (n->isan_spisize != COOKIE_SIZE * 2 ||
 	    pbs_left(pbs) < COOKIE_SIZE * 2) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: R_U_THERE_ACK has invalid SPI length (%d)",
-		       n->isan_spisize);
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: R_U_THERE_ACK has invalid SPI length (%d)",
+			  n->isan_spisize);
 		return STF_FAIL + PAYLOAD_MALFORMED;
 	}
 
 	if (!memeq(pbs->cur, p1st->st_ike_spis.initiator.bytes, COOKIE_SIZE)) {
 		/* RFC states we *SHOULD* check cookies, not MUST.  So invalid
 		   cookies are technically valid, as per Geoffrey Huang */
-		DBG(DBG_DPD,
-		    DBG_log("DPD: R_U_THERE_ACK has invalid icookie"));
+		dbg("DPD: R_U_THERE_ACK has invalid icookie");
 	}
 	pbs->cur += COOKIE_SIZE;
 
 	if (!memeq(pbs->cur, p1st->st_ike_spis.responder.bytes, COOKIE_SIZE)) {
 		/* RFC states we *SHOULD* check cookies, not MUST.  So invalid
 		   cookies are technically valid, as per Geoffrey Huang */
-		DBG(DBG_DPD,
-		    DBG_log("DPD: R_U_THERE_ACK has invalid rcookie"));
+		dbg("DPD: R_U_THERE_ACK has invalid rcookie");
 	}
 	pbs->cur += COOKIE_SIZE;
 
 	if (pbs_left(pbs) != sizeof(seqno)) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: R_U_THERE_ACK has invalid data length (%d)", (int) pbs_left(
-			       pbs));
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: R_U_THERE_ACK has invalid data length (%d)",
+			  (int) pbs_left(pbs));
 		return STF_FAIL + PAYLOAD_MALFORMED;
 	}
 
 	seqno = ntohl(*(uint32_t *)pbs->cur);
-	DBG(DBG_DPD,
-	    DBG_log("DPD: R_U_THERE_ACK, seqno received: %u expected: %u (state=#%lu)",
-		    seqno, p1st->st_dpd_expectseqno, p1st->st_serialno));
+	dbg("DPD: R_U_THERE_ACK, seqno received: %u expected: %u (state=#%lu)",
+	    seqno, p1st->st_dpd_expectseqno, p1st->st_serialno);
 
 	if (seqno == p1st->st_dpd_expectseqno) {
 		/* update the time stamp */
 		p1st->st_last_dpd = mononow();
 		p1st->st_dpd_expectseqno = 0;
 	} else if (!p1st->st_dpd_expectseqno) {
-		loglog(RC_LOG_SERIOUS,
-		       "DPD: unexpected R_U_THERE_ACK packet with sequence number %u",
-		       seqno);
+		log_state(RC_LOG_SERIOUS, p1st,
+			  "DPD: unexpected R_U_THERE_ACK packet with sequence number %u",
+			  seqno);
 		/* do not update time stamp, so we'll send a new one sooner */
 	}
 
@@ -546,7 +594,7 @@ stf_status dpd_inR(struct state *p1st,
 	 */
 	if (p1st->st_dpd_event != NULL &&
 	    p1st->st_dpd_event->ev_type == EVENT_DPD_TIMEOUT)
-		delete_dpd_event(p1st);
+		event_delete(EVENT_DPD, p1st);
 
 	return STF_IGNORE;
 }
@@ -561,8 +609,8 @@ stf_status dpd_inR(struct state *p1st,
  * @param st A state structure that is fully negotiated
  * @return void
  */
+
 void dpd_timeout(struct state *st)
 {
-	set_cur_state(st);
-	liveness_action(st->st_connection, st->st_ike_version);
+	dpd_action(st);
 }
