@@ -63,7 +63,7 @@
 #include "asn1.h"
 #include "pending.h"
 #include "ikev1_hash.h"
-#include "hostpair.h"
+#include "host_pair.h"
 #include "crypt_symkey.h"		/* for release_symkey() */
 
 #include "crypto.h"
@@ -71,6 +71,7 @@
 
 #include "ike_alg.h"
 #include "ike_alg_encrypt_ops.h"	/* XXX: oops */
+#include "ike_alg_hash.h"		/* for ike_alg_hash_sha1 */
 #include "kernel_alg.h"
 #include "plutoalg.h"
 #include "ikev1.h"
@@ -80,6 +81,7 @@
 #include "crypt_prf.h"
 #include "vendor.h"
 #include "nat_traversal.h"
+#include "ikev1_nat.h"
 #include "ikev1_dpd.h"
 #include "pluto_x509.h"
 #include "crypt_ke.h"
@@ -94,20 +96,21 @@
 # include "kernel_xfrm_interface.h"
 #endif
 #include "unpack.h"
+#include "ikev1_host_pair.h"
 
 /*
  * Initiate an Oakley Main Mode exchange.
  * --> HDR;SA
  * Note: this is not called from demux.c
  */
-/* extern initiator_function main_outI1; */	/* type assertion */
+
 void main_outI1(struct fd *whack_sock,
 		struct connection *c,
 		struct state *predecessor,
 		lset_t policy,
 		unsigned long try,
 		const threadtime_t *inception,
-		chunk_t sec_label)
+		shunk_t sec_label)
 {
 	struct ike_sa *ike = new_v1_istate(c, whack_sock);
 	struct state *st = &ike->sa;
@@ -119,10 +122,10 @@ void main_outI1(struct fd *whack_sock,
 	change_state(st, STATE_MAIN_I1);
 
 	if (HAS_IPSEC_POLICY(policy)) {
-		add_pending(whack_sock, ike, c, policy, 1,
-			    predecessor == NULL ?
-			    SOS_NOBODY : predecessor->st_serialno,
-			    sec_label, true /* part of initiate */);
+		add_v1_pending(whack_sock, ike, c, policy, 1,
+			       predecessor == NULL ?
+			       SOS_NOBODY : predecessor->st_serialno,
+			       sec_label, true /* part of initiate */);
 	}
 
 	if (predecessor == NULL) {
@@ -161,7 +164,7 @@ void main_outI1(struct fd *whack_sock,
 		uint8_t *sa_start = rbody.cur;
 
 		if (!ikev1_out_sa(&rbody, IKEv1_oakley_sadb(policy, c),
-				  st, TRUE, FALSE)) {
+				  st, true, false)) {
 			log_state(RC_LOG, st, "outsa fail");
 			return;
 		}
@@ -313,75 +316,6 @@ struct hash_signature v1_sign_hash_RSA(const struct connection *c,
 	return sig;
 }
 
-/*
- * Check a Main Mode RSA Signature against computed hash using RSA public
- * key k.
- *
- * As a side effect, on success, the public key is copied into the
- * state object to record the authenticator.
- *
- * Can fail because wrong public key is used or because hash disagrees.
- * We distinguish because diagnostics should also.
- *
- * The result is NULL if the Signature checked out.
- * Otherwise, the first character of the result indicates
- * how far along failure occurred.  A greater character signifies
- * greater progress.
- *
- * Classes:
- * 0	reserved for caller
- * 1	SIG length doesn't match key length -- wrong key
- * 2-8	malformed ECB after decryption -- probably wrong key
- * 9	decrypted hash != computed hash -- probably correct key
- * 10   NSS error
- * 11   NSS error
- * 12   NSS error
- *
- * Although the math should be the same for generating and checking signatures,
- * it is not: the knowledge of the private key allows more efficient (i.e.
- * different) computation for encryption.
- */
-static err_t try_RSA_signature_v1(const struct crypt_mac *hash,
-				const pb_stream *sig_pbs, struct pubkey *kr,
-				struct state *st,
-				const struct hash_desc *hash_algo_unused UNUSED /* for ikev2 only */)
-{
-	const uint8_t *sig_val = sig_pbs->cur;
-	size_t sig_len = pbs_left(sig_pbs);
-	const struct RSA_public_key *k = &kr->u.rsa;
-
-	/* decrypt the signature -- reversing RSA_sign_hash */
-	if (sig_len != kr->size) {
-		/* XXX notification: INVALID_KEY_INFORMATION */
-		return "1" "SIG length does not match public key length";
-	}
-
-	err_t ugh = RSA_signature_verify_nss(k, hash, sig_val,
-					     sig_len, 0 /* for ikev2 only */,
-					     st->st_logger);
-	if (ugh != NULL)
-		return ugh;
-
-	/*
-	 * Success: copy successful key into state.
-	 * There might be an old one if we previously aborted this
-	 * state transition.
-	 */
-	pubkey_delref(&st->st_peer_pubkey, HERE);
-	st->st_peer_pubkey = pubkey_addref(kr, HERE);
-
-	return NULL; /* happy happy */
-}
-
-static stf_status RSA_check_signature(struct state *st,
-				      struct crypt_mac *hash,
-				      const pb_stream *sig_pbs,
-				      enum ikev2_hash_algorithm hash_algo UNUSED /* for ikev2 only */)
-{
-	return check_signature_gen(st, hash, sig_pbs, 0 /* for ikev2 only */,
-				   &pubkey_type_rsa, try_RSA_signature_v1);
-}
-
 /*
  * encrypt message, sans fixed part of header
  * IV is fetched from st->st_new_iv and stored into st->st_iv.
@@ -410,7 +344,7 @@ bool ikev1_encrypt_message(pb_stream *pbs, struct state *st)
 
 		if (padding != 0) {
 			if (!out_zero(padding, pbs, "encryption padding"))
-				return FALSE;
+				return false;
 
 			enc_len += padding;
 		}
@@ -426,11 +360,11 @@ bool ikev1_encrypt_message(pb_stream *pbs, struct state *st)
 
 	/* close just before encrypting so NP backpatching isn't confused */
 	if (!ikev1_close_message(pbs, st))
-		return FALSE;
+		return false;
 
 	e->encrypt_ops->do_crypt(e, enc_start, enc_len,
 				 st->st_enc_key_nss,
-				 st->st_v1_new_iv.ptr, TRUE,
+				 st->st_v1_new_iv.ptr, true,
 				 st->st_logger);
 
 	update_iv(st);
@@ -438,7 +372,7 @@ bool ikev1_encrypt_message(pb_stream *pbs, struct state *st)
 		DBG_dump_hunk("next IV:", st->st_v1_iv);
 	}
 
-	return TRUE;
+	return true;
 }
 
 /*
@@ -474,11 +408,11 @@ bool ikev1_close_message(pb_stream *pbs, const struct state *st)
 	} else {
 		dbg("padding IKEv1 message with %zu bytes", padding);
 		if (!out_zero(padding, pbs, "message padding"))
-			return FALSE;
+			return false;
 	}
 
 	close_output_pbs(pbs);
-	return TRUE;
+	return true;
 }
 
 /*
@@ -520,8 +454,9 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 	}
 
 	/* random source ports are handled by find_host_connection */
-	c = find_host_connection(IKEv1, &md->iface->local_endpoint, &md->sender,
-				 LEMPTY, POLICY_AGGRESSIVE);
+	c = find_v1_host_connection(md->iface->ip_dev->id_address,
+				    endpoint_address(md->sender),
+				    LEMPTY, POLICY_AGGRESSIVE, NULL /* peer ID not known yet */);
 
 	if (c == NULL) {
 		lset_t policy = preparse_isakmp_sa_body(sa_pd->pbs);
@@ -545,8 +480,10 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 		 * but Food Groups kind of assumes one.
 		 */
 		{
-			struct connection *d = find_host_connection(IKEv1, &md->iface->local_endpoint, NULL,
-								    policy, POLICY_XAUTH | POLICY_AGGRESSIVE);
+			struct connection *d = find_v1_host_connection(md->iface->ip_dev->id_address,
+								       unset_address, policy,
+								       POLICY_XAUTH | POLICY_AGGRESSIVE,
+								       NULL /* peer ID not known yet */);
 
 			while (d != NULL) {
 				if (d->kind == CK_GROUP) {
@@ -565,14 +502,15 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 					 * Opportunistic or Shunt:
 					 * pick tightest match
 					 */
-					if (endpoint_in_selector(&md->sender, &d->spd.that.client) &&
-					    (c == NULL || selector_in_selector(&c->spd.that.client,
-									       &d->spd.that.client))) {
+					if (endpoint_in_selector(md->sender, d->spd.that.client) &&
+					    (c == NULL || selector_in_selector(c->spd.that.client,
+									       d->spd.that.client))) {
 						c = d;
 					}
 				}
-				d = find_next_host_connection(IKEv1, d->hp_next,
-							      policy, POLICY_XAUTH | POLICY_AGGRESSIVE);
+				d = find_next_v1_host_connection(d->hp_next,
+								 policy, POLICY_XAUTH | POLICY_AGGRESSIVE,
+								 NULL /* peer ID not known yet */);
 			}
 		}
 
@@ -603,30 +541,24 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 		connection_buf cib;
 		dbgl(md->md_logger, "instantiating "PRI_CONNECTION" for initial Main Mode message",
 		     pri_connection(c, &cib));
-		ip_address sender_address = endpoint_address(&md->sender);
+		ip_address sender_address = endpoint_address(md->sender);
 		c = rw_instantiate(c, &sender_address, NULL, NULL);
 	} else {
 		/*
-		 * we found a non-wildcard conn. double check if it needs
+		 * we found a non %any conn. double check if it needs
 		 * instantiation anyway (eg vnet=)
 		 */
-		if (c->kind == CK_TEMPLATE && c->spd.that.virt != NULL) {
+		if (c->kind == CK_TEMPLATE) {
 			dbgl(md->md_logger,
-			     "local endpoint has virt (vnet/vhost) set without wildcards - needs instantiation");
-			ip_address sender_address = endpoint_address(&md->sender);
-			c = rw_instantiate(c, &sender_address, NULL, NULL);
-		}
-		if (c->kind == CK_TEMPLATE && c->spd.that.has_id_wildcards) {
-			dbgl(md->md_logger,
-			     "remote end has wildcard ID, needs instantiation");
-			ip_address sender_address = endpoint_address(&md->sender);
+			     "local endpoint needs instantiation");
+			ip_address sender_address = endpoint_address(md->sender);
 			c = rw_instantiate(c, &sender_address, NULL, NULL);
 		}
 	}
 
 	/* Set up state */
 	struct ike_sa *ike = new_v1_rstate(c, md);
-	struct state *st = md->st = &ike->sa;
+	struct state *st = md->v1_st = &ike->sa;
 
 	passert(!st->st_oakley.doing_xauth);
 
@@ -648,7 +580,7 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 	if (c->kind == CK_INSTANCE) {
 		endpoint_buf b;
 		log_state(RC_LOG, st, "responding to Main Mode from unknown peer %s",
-			  str_sensitive_endpoint(&md->sender, &b));
+			  str_endpoint_sensitive(&md->sender, &b));
 	} else {
 		log_state(RC_LOG, st, "responding to Main Mode");
 	}
@@ -693,7 +625,7 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 	/* SA body in and out */
 	RETURN_STF_FAILURE(parse_isakmp_sa_body(&sa_pd->pbs,
 						&sa_pd->payload.sa,
-						&r_sa_pbs, FALSE, st));
+						&r_sa_pbs, false, st));
 
 	/* send Vendor IDs */
 	if (!out_vid_set(&rbody, c))
@@ -709,8 +641,8 @@ stf_status main_inI1_outR1(struct state *unused_st UNUSED,
 		return STF_INTERNAL_ERROR;
 
 	/* save initiator SA for HASH */
-	free_chunk_content(&st->st_p1isa);
-	st->st_p1isa = clone_hunk(pbs_in_as_shunk(&sa_pd->pbs), "sa in main_inI1_outR1()");
+	replace_chunk(&st->st_p1isa,
+		      clone_pbs_in_as_chunk(&sa_pd->pbs, "sa in main_inI1_outR1()"));
 
 	return STF_OK;
 }
@@ -739,7 +671,7 @@ stf_status main_inR1_outI2(struct state *st, struct msg_digest *md)
 
 		RETURN_STF_FAILURE(parse_isakmp_sa_body(&sapd->pbs,
 							&sapd->payload.sa,
-							NULL, TRUE, st));
+							NULL, true, st));
 	}
 
 	if (libreswan_fipsmode() && st->st_oakley.ta_prf == NULL) {
@@ -783,7 +715,7 @@ static stf_status main_inR1_outI2_continue(struct state *st,
 	 * depends on the type of Auth (eventually).
 	 */
 	struct pbs_out rbody;
-	ikev1_init_pbs_out_from_md_hdr(md, FALSE,
+	ikev1_init_pbs_out_from_md_hdr(md, false,
 				       &reply_stream, reply_buffer, sizeof(reply_buffer),
 				       &rbody, st->st_logger);
 
@@ -865,7 +797,7 @@ stf_status main_inI2_outR2(struct state *st, struct msg_digest *md)
 	ikev1_decode_cr(md, st->st_logger);
 
 	if (st->st_requested_ca != NULL)
-		st->hidden_variables.st_got_certrequest = TRUE;
+		st->hidden_variables.st_got_certrequest = true;
 
 	ikev1_natd_init(st, md);
 
@@ -931,7 +863,7 @@ static stf_status main_inI2_outR2_continue1(struct state *st,
 	}
 
 	/* send CR if auth is RSA and no preloaded RSA public key exists*/
-	bool send_cr = FALSE;
+	bool send_cr = false;
 
 	/* Build output packet HDR;KE;Nr */
 
@@ -941,7 +873,7 @@ static stf_status main_inI2_outR2_continue1(struct state *st,
 
 	/* HDR out */
 	struct pbs_out rbody;
-	ikev1_init_pbs_out_from_md_hdr(md, FALSE,
+	ikev1_init_pbs_out_from_md_hdr(md, false,
 				       &reply_stream, reply_buffer, sizeof(reply_buffer),
 				       &rbody, st->st_logger);
 
@@ -983,11 +915,11 @@ static stf_status main_inI2_outR2_continue1(struct state *st,
 					if (!ikev1_build_and_ship_CR(CERT_X509_SIGNATURE,
 								     gn->name,
 								     &rbody)) {
-						free_generalNames(ca, FALSE);
+						free_generalNames(ca, false);
 						return STF_INTERNAL_ERROR;
 					}
 				}
-				free_generalNames(ca, FALSE);
+				free_generalNames(ca, false);
 			} else {
 				if (!ikev1_build_and_ship_CR(CERT_X509_SIGNATURE,
 							     EMPTY_CHUNK,
@@ -1026,7 +958,7 @@ static stf_status main_inI2_outR2_continue1(struct state *st,
 	submit_dh_shared_secret(st, st->st_gi/*responder needs initiator's KE*/,
 				main_inI2_outR2_continue2, HERE);
 	/* we are calculating in the background, so it doesn't count */
-	dbg("#%lu %s:%u st->st_calculating = FALSE;", st->st_serialno, __func__, __LINE__);
+	dbg("#%lu %s:%u st->st_calculating = false;", st->st_serialno, __func__, __LINE__);
 	st->st_v1_offloaded_task_in_background = true;
 
 	return STF_OK;
@@ -1061,29 +993,29 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 	calc_v1_skeyid_and_iv(st);
 
 	struct pbs_out rbody[1]; /* hack */
-	ikev1_init_pbs_out_from_md_hdr(md, TRUE,
+	ikev1_init_pbs_out_from_md_hdr(md, true,
 				       &reply_stream, reply_buffer, sizeof(reply_buffer),
 				       rbody, st->st_logger);
 
 	const struct connection *c = st->st_connection;
-	const cert_t mycert = c->spd.this.cert;
+	const struct cert *mycert = c->spd.this.cert.nss_cert != NULL ? &c->spd.this.cert : NULL;
 
 	/* decode certificate requests */
 	ikev1_decode_cr(md, st->st_logger);
 
 	if (st->st_requested_ca != NULL)
-		st->hidden_variables.st_got_certrequest = TRUE;
+		st->hidden_variables.st_got_certrequest = true;
 
 	/*
 	 * send certificate if we have one and auth is RSA, and we were
 	 * told we can send one if asked, and we were asked, or we were told
 	 * to always send one.
 	 */
-	bool send_cert = st->st_oakley.auth == OAKLEY_RSA_SIG &&
-		mycert.ty != CERT_NONE && mycert.u.nss_cert != NULL &&
-		((c->spd.this.sendcert == CERT_SENDIFASKED &&
-		  st->hidden_variables.st_got_certrequest) ||
-		 c->spd.this.sendcert == CERT_ALWAYSSEND);
+	bool send_cert = (st->st_oakley.auth == OAKLEY_RSA_SIG &&
+			  mycert != NULL &&
+			  ((c->spd.this.sendcert == CERT_SENDIFASKED &&
+			    st->hidden_variables.st_got_certrequest) ||
+			   c->spd.this.sendcert == CERT_ALWAYSSEND));
 
 	bool send_authcerts = (send_cert &&
 			  c->send_ca != CA_SEND_NONE);
@@ -1097,19 +1029,16 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 	int chain_len = 0;
 
 	if (send_authcerts) {
-		chain_len = get_auth_chain(auth_chain, MAX_CA_PATH_LEN,
-					   mycert.u.nss_cert,
+		chain_len = get_auth_chain(auth_chain, MAX_CA_PATH_LEN, mycert,
 					   c->send_ca == CA_SEND_ALL);
 		if (chain_len == 0)
-			send_authcerts = FALSE;
+			send_authcerts = false;
 	}
 
-	doi_log_cert_thinking(st->st_oakley.auth,
-			mycert.ty,
-			c->spd.this.sendcert,
-			st->hidden_variables.st_got_certrequest,
-			send_cert,
-			send_authcerts);
+	doi_log_cert_thinking(st->st_oakley.auth, cert_ike_type(mycert),
+			      c->spd.this.sendcert,
+			      st->hidden_variables.st_got_certrequest,
+			      send_cert, send_authcerts);
 
 	/*
 	 * send certificate request, if we don't have a preloaded RSA
@@ -1169,7 +1098,7 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 				&isakmp_ipsec_identification_desc,
 				rbody,
 				&id_pbs) ||
-		    !pbs_out_hunk(id_b, &id_pbs, "my identity")) {
+		    !out_hunk(id_b, &id_pbs, "my identity")) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
@@ -1180,13 +1109,13 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 	/* CERT out */
 	if (send_cert && impair.send_pkcs7_thingie) {
 		log_state(RC_LOG, st, "IMPAIR: sending cert as pkcs7 blob");
-		SECItem *pkcs7 = nss_pkcs7_blob(mycert.u.nss_cert, send_authcerts);
+		SECItem *pkcs7 = nss_pkcs7_blob(mycert, send_authcerts);
 		if (!pexpect(pkcs7 != NULL)) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
 		if (!ikev1_ship_CERT(CERT_PKCS7_WRAPPED_X509,
-				     same_secitem_as_chunk(*pkcs7),
+				     same_secitem_as_shunk(*pkcs7),
 				     rbody)) {
 			SECITEM_FreeItem(pkcs7, PR_TRUE);
 			free_auth_chain(auth_chain, chain_len);
@@ -1195,9 +1124,7 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 	} else if (send_cert) {
 		log_state(RC_LOG, st, "I am sending my cert");
 
-		if (!ikev1_ship_CERT(mycert.ty,
-				   get_dercert_from_nss_cert(mycert.u.nss_cert),
-				   rbody)) {
+		if (!ikev1_ship_CERT(cert_ike_type(mycert), cert_der(mycert), rbody)) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
@@ -1208,7 +1135,7 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 			if (!ikev1_ship_chain(auth_chain,
 					      chain_len,
 					      rbody,
-					      mycert.ty)) {
+					      cert_ike_type(mycert))) {
 				free_auth_chain(auth_chain, chain_len);
 				return STF_INTERNAL_ERROR;
 			}
@@ -1222,7 +1149,7 @@ static stf_status main_inR2_outI3_continue(struct state *st,
 	/* CR out */
 	if (send_cr) {
 		log_state(RC_LOG, st, "I am sending a certificate request");
-		if (!ikev1_build_and_ship_CR(mycert.ty,
+		if (!ikev1_build_and_ship_CR(cert_ike_type(mycert),
 					     c->spd.that.ca,
 					     rbody))
 			return STF_INTERNAL_ERROR;
@@ -1322,9 +1249,9 @@ stf_status main_inR2_outI3(struct state *st, struct msg_digest *md)
  * XXX: This is used by aggressive mode too, move to ikev1.c ???
  */
 stf_status oakley_id_and_auth(struct msg_digest *md, bool initiator,
-			bool aggrmode)
+			      bool aggrmode)
 {
-	struct state *st = md->st;
+	struct state *st = md->v1_st;
 	stf_status r = STF_OK;
 
 	/*
@@ -1332,7 +1259,7 @@ stf_status oakley_id_and_auth(struct msg_digest *md, bool initiator,
 	 * Note: ikev1_decode_peer_id may switch the connection being used!
 	 * But only if we are a Main Mode Responder.
 	 */
-	if (!st->st_peer_alt_id) {
+	if (!st->st_v1_peer_alt_id) {
 		if (!ikev1_decode_peer_id(md, initiator, aggrmode)) {
 			dbg("Peer ID failed to decode");
 			return STF_FAIL + INVALID_ID_INFORMATION;
@@ -1342,7 +1269,7 @@ stf_status oakley_id_and_auth(struct msg_digest *md, bool initiator,
 	/*
 	 * process any CERT payloads if aggrmode
 	 */
-	if (!st->st_peer_alt_id) {
+	if (!st->st_v1_peer_alt_id) {
 		if (!v1_verify_certs(md)) {
 			return STF_FAIL + INVALID_ID_INFORMATION;
 		}
@@ -1396,12 +1323,17 @@ stf_status oakley_id_and_auth(struct msg_digest *md, bool initiator,
 
 	case OAKLEY_RSA_SIG:
 	{
-		r = RSA_check_signature(st, &hash,
-					&md->chain[ISAKMP_NEXT_SIG]->pbs, 0 /* for ikev2 only*/);
-		if (r != STF_OK) {
+		shunk_t signature = pbs_in_left_as_shunk(&md->chain[ISAKMP_NEXT_SIG]->pbs);
+		diag_t d = authsig_and_log_using_pubkey(ike_sa(st, HERE), &hash, signature,
+							&ike_alg_hash_sha1, /*always*/
+							&pubkey_type_rsa,
+							authsig_using_RSA_pubkey);
+		if (d != NULL) {
+			llog_diag(RC_LOG_SERIOUS, st->st_logger, &d, "%s", "");
 			dbg("received '%s' message SIG_%s data did not match computed value",
 			    aggrmode ? "Aggr" : "Main",
 			    initiator ? "R" : "I" /*reverse*/);
+			r = STF_FAIL + INVALID_KEY_INFORMATION;
 		}
 		break;
 	}
@@ -1425,8 +1357,8 @@ stf_status oakley_id_and_auth(struct msg_digest *md, bool initiator,
 
 stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 {
-	pexpect(st == md->st);
-	st = md->st;
+	pexpect(st == md->v1_st);
+	st = md->v1_st;
 
 	/* handle case where NSS balked at generating DH */
 	if (st->st_dh_shared_secret == NULL)
@@ -1443,20 +1375,20 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 	 * since we are a Main Mode Responder.
 	 */
 	{
-		stf_status r = oakley_id_and_auth(md, FALSE, FALSE);
+		stf_status r = oakley_id_and_auth(md, false, false);
 		if (r != STF_OK)
 			return r;
 	}
 	struct connection *c = st->st_connection;
 
 	/* send certificate if we have one and auth is RSA */
-	cert_t mycert = c->spd.this.cert;
+	const struct cert *mycert = c->spd.this.cert.nss_cert != NULL ? &c->spd.this.cert : NULL;
 
-	bool send_cert = st->st_oakley.auth == OAKLEY_RSA_SIG &&
-		mycert.ty != CERT_NONE && mycert.u.nss_cert != NULL &&
-		((c->spd.this.sendcert == CERT_SENDIFASKED &&
-		  st->hidden_variables.st_got_certrequest) ||
-		 c->spd.this.sendcert == CERT_ALWAYSSEND);
+	bool send_cert = (st->st_oakley.auth == OAKLEY_RSA_SIG &&
+			  mycert != NULL &&
+			  ((c->spd.this.sendcert == CERT_SENDIFASKED &&
+			    st->hidden_variables.st_got_certrequest) ||
+			   c->spd.this.sendcert == CERT_ALWAYSSEND));
 
 	bool send_authcerts = (send_cert &&
 			  c->send_ca != CA_SEND_NONE);
@@ -1470,19 +1402,16 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 	int chain_len = 0;
 
 	if (send_authcerts) {
-		chain_len = get_auth_chain(auth_chain, MAX_CA_PATH_LEN,
-					   mycert.u.nss_cert,
+		chain_len = get_auth_chain(auth_chain, MAX_CA_PATH_LEN, mycert,
 					   c->send_ca == CA_SEND_ALL);
 		if (chain_len == 0)
-			send_authcerts = FALSE;
+			send_authcerts = false;
 	}
 
-	doi_log_cert_thinking(st->st_oakley.auth,
-			mycert.ty,
-			c->spd.this.sendcert,
-			st->hidden_variables.st_got_certrequest,
-			send_cert,
-			send_authcerts);
+	doi_log_cert_thinking(st->st_oakley.auth, cert_ike_type(mycert),
+			      c->spd.this.sendcert,
+			      st->hidden_variables.st_got_certrequest,
+			      send_cert, send_authcerts);
 
 	/*
 	 * Build output packet HDR*;IDir;HASH/SIG_R
@@ -1501,7 +1430,7 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 	 * be first payload.
 	 */
 	struct pbs_out rbody;
-	ikev1_init_pbs_out_from_md_hdr(md, TRUE,
+	ikev1_init_pbs_out_from_md_hdr(md, true,
 				       &reply_stream, reply_buffer, sizeof(reply_buffer),
 				       &rbody, st->st_logger);
 
@@ -1520,7 +1449,7 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 		struct isakmp_ipsec_id id_hd = build_v1_id_payload(&c->spd.this, &id_b);
 		if (!out_struct(&id_hd, &isakmp_ipsec_identification_desc,
 					&rbody, &r_id_pbs) ||
-		    !pbs_out_hunk(id_b, &r_id_pbs, "my identity")) {
+		    !out_hunk(id_b, &r_id_pbs, "my identity")) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
@@ -1531,13 +1460,13 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 	/* CERT out, if we have one */
 	if (send_cert && impair.send_pkcs7_thingie) {
 		log_state(RC_LOG, st, "IMPAIR: sending cert as pkcs7 blob");
-		SECItem *pkcs7 = nss_pkcs7_blob(mycert.u.nss_cert, send_authcerts);
+		SECItem *pkcs7 = nss_pkcs7_blob(mycert, send_authcerts);
 		if (!pexpect(pkcs7 != NULL)) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
 		if (!ikev1_ship_CERT(CERT_PKCS7_WRAPPED_X509,
-				     same_secitem_as_chunk(*pkcs7),
+				     same_secitem_as_shunk(*pkcs7),
 				     &rbody)) {
 			SECITEM_FreeItem(pkcs7, PR_TRUE);
 			free_auth_chain(auth_chain, chain_len);
@@ -1545,9 +1474,7 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 		}
 	} else if (send_cert) {
 		log_state(RC_LOG, st, "I am sending my cert");
-		if (!ikev1_ship_CERT(mycert.ty,
-				     get_dercert_from_nss_cert(mycert.u.nss_cert),
-				     &rbody)) {
+		if (!ikev1_ship_CERT(cert_ike_type(mycert), cert_der(mycert), &rbody)) {
 			free_auth_chain(auth_chain, chain_len);
 			return STF_INTERNAL_ERROR;
 		}
@@ -1555,7 +1482,7 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 		if (send_authcerts) {
 			log_state(RC_LOG, st, "I am sending a CA cert chain");
 			if (!ikev1_ship_chain(auth_chain, chain_len,
-					      &rbody, mycert.ty)) {
+					      &rbody, cert_ike_type(mycert))) {
 				free_auth_chain(auth_chain, chain_len);
 				return STF_INTERNAL_ERROR;
 			}
@@ -1613,20 +1540,20 @@ stf_status main_inI3_outR3(struct state *st, struct msg_digest *md)
 	 */
 
 	if (c->remotepeertype == CISCO &&
-	    c->newest_isakmp_sa != SOS_NOBODY &&
+	    c->newest_ike_sa != SOS_NOBODY &&
 	    c->spd.this.xauth_client) {
 		dbg("Skipping XAUTH for rekey for Cisco Peer compatibility.");
-		st->hidden_variables.st_xauth_client_done = TRUE;
-		st->st_oakley.doing_xauth = FALSE;
+		st->hidden_variables.st_xauth_client_done = true;
+		st->st_oakley.doing_xauth = false;
 
 		if (c->spd.this.modecfg_client) {
 			dbg("Skipping ModeCFG for rekey for Cisco Peer compatibility.");
-			st->hidden_variables.st_modecfg_vars_set = TRUE;
-			st->hidden_variables.st_modecfg_started = TRUE;
+			st->hidden_variables.st_modecfg_vars_set = true;
+			st->hidden_variables.st_modecfg_started = true;
 		}
 	}
 
-	IKE_SA_established(pexpect_ike_sa(st));
+	ISAKMP_SA_established(pexpect_ike_sa(st));
 #ifdef USE_XFRM_INTERFACE
 	if (c->xfrmi != NULL && c->xfrmi->if_id != 0)
 		if (add_xfrmi(c, st->st_logger))
@@ -1655,7 +1582,7 @@ stf_status main_inR3(struct state *st, struct msg_digest *md)
 	 * because we are the Responder.
 	 */
 	{
-		stf_status r = oakley_id_and_auth(md, TRUE, FALSE);
+		stf_status r = oakley_id_and_auth(md, true, false);
 		if (r != STF_OK)
 			return r;
 	}
@@ -1668,20 +1595,20 @@ stf_status main_inR3(struct state *st, struct msg_digest *md)
 	 * are not supposed to be performed again during rekey
 	 */
 	if (c->remotepeertype == CISCO &&
-		c->newest_isakmp_sa != SOS_NOBODY &&
+		c->newest_ike_sa != SOS_NOBODY &&
 		c->spd.this.xauth_client) {
 		dbg("Skipping XAUTH for rekey for Cisco Peer compatibility.");
-		st->hidden_variables.st_xauth_client_done = TRUE;
-		st->st_oakley.doing_xauth = FALSE;
+		st->hidden_variables.st_xauth_client_done = true;
+		st->st_oakley.doing_xauth = false;
 
 		if (c->spd.this.modecfg_client) {
 			dbg("Skipping ModeCFG for rekey for Cisco Peer compatibility.");
-			st->hidden_variables.st_modecfg_vars_set = TRUE;
-			st->hidden_variables.st_modecfg_started = TRUE;
+			st->hidden_variables.st_modecfg_vars_set = true;
+			st->hidden_variables.st_modecfg_started = true;
 		}
 	}
 
-	IKE_SA_established(pexpect_ike_sa(st));
+	ISAKMP_SA_established(pexpect_ike_sa(st));
 #ifdef USE_XFRM_INTERFACE
 	if (c->xfrmi != NULL && c->xfrmi->if_id != 0)
 		if (add_xfrmi(c, st->st_logger))
@@ -1831,7 +1758,7 @@ static void send_notification(struct logger *logger,
 			llog(RC_LOG, logger, "too many (%d) malformed payloads. Deleting state",
 				    sndst->hidden_variables.st_malformed_sent);
 			delete_state(sndst);
-			/* note: no md->st to clear */
+			/* note: no md->v1_st to clear */
 			return;
 		}
 
@@ -1862,7 +1789,7 @@ static void send_notification(struct logger *logger,
 		break;
 	}
 
-	if (encst != NULL && !IS_ISAKMP_ENCRYPTED(encst->st_state->kind))
+	if (encst != NULL && !IS_V1_ISAKMP_ENCRYPTED(encst->st_state->kind))
 		encst = NULL;
 
 	{
@@ -1942,7 +1869,7 @@ static void send_notification(struct logger *logger,
 
 		save_iv(encst, old_iv);
 
-		if (!IS_ISAKMP_SA_ESTABLISHED(encst->st_state)) {
+		if (!IS_V1_ISAKMP_SA_ESTABLISHED(encst)) {
 			update_iv(encst);
 		}
 		init_phase2_iv(encst, &msgid);
@@ -1966,11 +1893,10 @@ void send_notification_from_state(struct state *st, enum state_kind from_state,
 	if (from_state == STATE_UNDEFINED)
 		from_state = st->st_state->kind;
 
-	if (IS_QUICK(from_state)) {
-		p1st = find_phase1_state(st->st_connection,
-					ISAKMP_SA_ESTABLISHED_STATES);
+	if (IS_V1_QUICK(from_state)) {
+		p1st = find_phase1_state(st->st_connection, V1_ISAKMP_SA_ESTABLISHED_STATES);
 		if ((p1st == NULL) ||
-			(!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state))) {
+			(!IS_V1_ISAKMP_SA_ESTABLISHED(p1st))) {
 			log_state(RC_LOG_SERIOUS, st,
 				  "no Phase1 state for Quick mode notification");
 			return;
@@ -1978,7 +1904,7 @@ void send_notification_from_state(struct state *st, enum state_kind from_state,
 		send_notification(st->st_logger, st, type, p1st, generate_msgid(p1st),
 				  st->st_ike_spis.initiator.bytes, st->st_ike_spis.responder.bytes,
 				  PROTO_ISAKMP);
-	} else if (IS_ISAKMP_ENCRYPTED(from_state)) {
+	} else if (IS_V1_ISAKMP_ENCRYPTED(from_state)) {
 		send_notification(st->st_logger, st, type, st, generate_msgid(st),
 				st->st_ike_spis.initiator.bytes, st->st_ike_spis.responder.bytes,
 				PROTO_ISAKMP);
@@ -2078,32 +2004,35 @@ void send_v1_delete(struct state *st)
 	struct state *p1st;
 	ip_said said[EM_MAXRELSPIS];
 	ip_said *ns = said;
-	bool isakmp_sa = FALSE;
+	bool isakmp_sa = false;
 
 	/* If there are IPsec SA's related to this state struct... */
 	if (IS_IPSEC_SA_ESTABLISHED(st)) {
 		/* Find their phase1 state object */
-		p1st = find_phase1_state(st->st_connection,
-					ISAKMP_SA_ESTABLISHED_STATES);
+		p1st = find_phase1_state(st->st_connection, V1_ISAKMP_SA_ESTABLISHED_STATES);
 		if (p1st == NULL) {
 			dbg("no Phase 1 state for Delete");
 			return;
 		}
 
 		if (st->st_ah.present) {
-			*ns = said3(&st->st_connection->spd.this.host_addr, st->st_ah.our_spi, &ip_protocol_ah);
+			*ns = said_from_address_protocol_spi(st->st_connection->spd.this.host_addr,
+							     &ip_protocol_ah,
+							     st->st_ah.our_spi);
 			ns++;
 		}
 		if (st->st_esp.present) {
-			*ns = said3(&st->st_connection->spd.this.host_addr, st->st_esp.our_spi, &ip_protocol_esp);
+			*ns = said_from_address_protocol_spi(st->st_connection->spd.this.host_addr,
+							     &ip_protocol_esp,
+							     st->st_esp.our_spi);
 			ns++;
 		}
 
 		passert(ns != said); /* there must be some SAs to delete */
-	} else if (IS_ISAKMP_SA_ESTABLISHED(st->st_state)) {
+	} else if (IS_V1_ISAKMP_SA_ESTABLISHED(st)) {
 		/* or ISAKMP SA's... */
 		p1st = st;
-		isakmp_sa = TRUE;
+		isakmp_sa = true;
 	} else {
 		return; /* nothing to do */
 	}
@@ -2157,10 +2086,11 @@ void send_v1_delete(struct state *st)
 		while (ns != said) {
 			pb_stream del_pbs;
 			ns--;
+			const struct ip_protocol *proto = said_protocol(*ns);
 			struct isakmp_delete isad = {
 				.isad_doi = ISAKMP_DOI_IPSEC,
 				.isad_spisize = sizeof(ipsec_spi_t),
-				.isad_protoid = ns->proto->ikev1_protocol_id,
+				.isad_protoid = proto->ikev1_protocol_id,
 				.isad_nospi = 1,
 			};
 
@@ -2231,31 +2161,31 @@ void send_v1_delete(struct state *st)
 bool accept_delete(struct msg_digest *md,
 		struct payload_digest *p)
 {
-	struct state *st = md->st;
+	struct state *st = md->v1_st;
 	struct isakmp_delete *d = &(p->payload.delete);
 	size_t sizespi;
 	int i;
-	bool self_delete = FALSE;
+	bool self_delete = false;
 
 	/* We only listen to encrypted notifications */
 	if (!md->encrypted) {
 		log_state(RC_LOG_SERIOUS, st,
 			  "ignoring Delete SA payload: not encrypted");
-		return FALSE;
+		return false;
 	}
 
 	/* If there is no SA related to this request, but it was encrypted */
-	if (!IS_ISAKMP_SA_ESTABLISHED(st->st_state)) {
+	if (!IS_V1_ISAKMP_SA_ESTABLISHED(st)) {
 		/* can't happen (if msg is encrypt), but just to be sure */
 		log_state(RC_LOG_SERIOUS, st,
 			  "ignoring Delete SA payload: ISAKMP SA not established");
-		return FALSE;
+		return false;
 	}
 
 	if (d->isad_nospi == 0) {
 		log_state(RC_LOG_SERIOUS, st,
 			  "ignoring Delete SA payload: no SPI");
-		return FALSE;
+		return false;
 	}
 
 	switch (d->isad_protoid) {
@@ -2270,20 +2200,24 @@ bool accept_delete(struct msg_digest *md,
 
 	case PROTO_IPCOMP:
 		/* nothing interesting to delete */
-		return FALSE;
+		return false;
 
 	default:
+	{
+		esb_buf b;
 		log_state(RC_LOG_SERIOUS, st,
 			  "ignoring Delete SA payload: unknown Protocol ID (%s)",
-			  enum_show(&ikev1_protocol_names, d->isad_protoid));
+			  enum_show(&ikev1_protocol_names, d->isad_protoid, &b));
 		return false;
 	}
+	}
 
 	if (d->isad_spisize != sizespi) {
+		esb_buf b;
 		log_state(RC_LOG_SERIOUS, st,
 			  "ignoring Delete SA payload: bad SPI size (%d) for %s",
 			  d->isad_spisize,
-			  enum_show(&ikev1_protocol_names, d->isad_protoid));
+			  enum_show(&ikev1_protocol_names, d->isad_protoid, &b));
 		return false;
 	}
 
@@ -2303,13 +2237,13 @@ bool accept_delete(struct msg_digest *md,
 
 			d = pbs_in_raw(&p->pbs, &cookies.initiator, COOKIE_SIZE, "iCookie");
 			if (d != NULL) {
-				log_diag(RC_LOG, st->st_logger, &d, "%s", "");
+				llog_diag(RC_LOG, st->st_logger, &d, "%s", "");
 				return false;
 			}
 
 			d = pbs_in_raw(&p->pbs, &cookies.responder, COOKIE_SIZE, "rCookie");
 			if (d != NULL) {
-				log_diag(RC_LOG, st->st_logger, &d, "%s", "");
+				llog_diag(RC_LOG, st->st_logger, &d, "%s", "");
 				return false;
 			}
 
@@ -2330,7 +2264,7 @@ bool accept_delete(struct msg_digest *md,
 				 * remember this for later:
 				 * we need st to do any remaining deletes
 				 */
-				self_delete = TRUE;
+				self_delete = true;
 			} else {
 				/* note: this code is cloned for handling self_delete */
 				log_state(RC_LOG_SERIOUS, st, "received Delete SA payload: deleting ISAKMP State #%lu",
@@ -2348,7 +2282,7 @@ bool accept_delete(struct msg_digest *md,
 			ipsec_spi_t spi;	/* network order */
 			diag_t dt = pbs_in_raw(&p->pbs, &spi, sizeof(spi), "SPI");
 			if (dt != NULL) {
-				log_diag(RC_LOG, st->st_logger, &dt, "%s", "");
+				llog_diag(RC_LOG, st->st_logger, &dt, "%s", "");
 				return false;
 			}
 
@@ -2360,17 +2294,17 @@ bool accept_delete(struct msg_digest *md,
 
 			passert(dst != st);	/* st is an IKE SA */
 			if (dst == NULL) {
+				esb_buf b;
 				log_state(RC_LOG_SERIOUS, st,
 					  "ignoring Delete SA payload: %s SA(0x%08" PRIx32 ") not found (maybe expired)",
-					  enum_show(&ikev1_protocol_names,
-						    d->isad_protoid),
+					  enum_show(&ikev1_protocol_names, d->isad_protoid, &b),
 					  ntohl(spi));
 			} else {
 				if (bogus) {
+					esb_buf b;
 					log_state(RC_LOG_SERIOUS, st,
 						  "warning: Delete SA payload: %s SA(0x%08" PRIx32 ") is our own SPI (bogus implementation) - deleting anyway",
-						  enum_show(&ikev1_protocol_names,
-							    d->isad_protoid),
+						  enum_show(&ikev1_protocol_names, d->isad_protoid, &b),
 						  ntohl(spi));
 				}
 
@@ -2402,8 +2336,8 @@ bool accept_delete(struct msg_digest *md,
 						  ntohl(spi),
 						  dst->st_serialno);
 					delete_state(dst);
-					if (md->st == dst)
-						md->st = NULL;
+					if (md->v1_st == dst)
+						md->v1_st = NULL;
 				}
 
 				if (rc->newest_ipsec_sa == SOS_NOBODY) {
@@ -2412,9 +2346,8 @@ bool accept_delete(struct msg_digest *md,
 					if (!shared_phase1_connection(rc)) {
 						flush_pending_by_connection(rc);
 						/* why loop? there can be only one IKE SA, just delete_state(st) ? */
-						delete_states_by_connection(rc, FALSE,
-									    null_fd/*no-whack?*/);
-						md->st = NULL;
+						delete_states_by_connection(rc, /*relations*/false);
+						md->v1_st = NULL;
 					}
 				}
 			}
@@ -2427,7 +2360,7 @@ bool accept_delete(struct msg_digest *md,
 /* now it is safe to delete our sponsor */
 void accept_self_delete(struct msg_digest *md)
 {
-	struct state *st = md->st;
+	struct state *st = md->v1_st;
 
 	/* note: this code is cloned from handling ISAKMP non-self_delete */
 	log_state(RC_LOG_SERIOUS, st, "received Delete SA payload: self-deleting ISAKMP State #%lu",
@@ -2437,5 +2370,5 @@ void accept_self_delete(struct msg_digest *md)
 		v1_maybe_natify_initiator_endpoints(st, HERE);
 	}
 	delete_state(st);
-	md->st = st = NULL;
+	md->v1_st = st = NULL;
 }
