@@ -20,6 +20,8 @@ struct spd_route;
 struct crypt_mac;
 struct hash_desc;
 struct payload_digest;
+struct ikev2_ipseckey_dns;
+enum payload_security;
 
 typedef stf_status crypto_transition_fn(struct state *st, struct msg_digest *md,
 					struct pluto_crypto_req *r);
@@ -28,66 +30,17 @@ void ikev2_process_packet(struct msg_digest *mdp);
 void ikev2_process_state_packet(struct ike_sa *ike, struct state *st,
 				struct msg_digest *mdp);
 
-/* extern initiator_function ikev2_parent_outI1; */
-extern void ikev2_parent_outI1(struct fd *whack_sock,
-			      struct connection *c,
-			      struct state *predecessor,
-			      lset_t policy,
-			      unsigned long try,
-			      const threadtime_t *inception,
-			      chunk_t sec_label);
-
-extern void log_ipsec_sa_established(const char *m, const struct state *st);
+typedef stf_status ikev2_state_transition_fn(struct ike_sa *ike,
+					     struct child_sa *child, /* could be NULL */
+					     struct msg_digest *md /* could be NULL */);
 
 extern void complete_v2_state_transition(struct state *st,
 					 struct msg_digest *mdp,
 					 stf_status result);
 
-typedef stf_status ikev2_state_transition_fn(struct ike_sa *ike,
-					     struct child_sa *child, /* could be NULL */
-					     struct msg_digest *md /* could be NULL */);
-
-extern ikev2_state_transition_fn process_encrypted_informational_ikev2;
-
-extern ikev2_state_transition_fn ikev2_child_ike_inIoutR;
-extern ikev2_state_transition_fn ikev2_child_ike_inR;
-extern ikev2_state_transition_fn ikev2_child_inR;
-extern ikev2_state_transition_fn ikev2_child_inIoutR;
-
-extern ikev2_state_transition_fn ikev2_parent_inI1outR1;
-extern ikev2_state_transition_fn ikev2_auth_initiator_process_failure_notification;
-extern ikev2_state_transition_fn ikev2_auth_initiator_process_unknown_notification;
-extern ikev2_state_transition_fn ikev2_ike_sa_process_intermediate_request_no_skeyid;
-extern ikev2_state_transition_fn ikev2_ike_sa_process_auth_request_no_skeyid;
-extern ikev2_state_transition_fn ikev2_ike_sa_process_auth_request;
-extern ikev2_state_transition_fn ikev2_parent_inR1outI2;
-extern ikev2_state_transition_fn ikev2_parent_inR2;
-
 void schedule_reinitiate_v2_ike_sa_init(struct ike_sa *ike,
 					stf_status (*resume)(struct ike_sa *ike));
 
-bool record_v2_IKE_SA_INIT_request(struct ike_sa *ike);
-extern ikev2_state_transition_fn process_IKE_SA_INIT_v2N_INVALID_KE_PAYLOAD_response;
-
-extern void ikev2_initiate_child_sa(struct pending *p);
-
-void ikev2_rekey_ike_start(struct ike_sa *ike);
-
-extern void ikev2_child_outI(struct state *st);
-
-extern stf_status ikev2_parent_inI2outR2_id_tail(struct msg_digest * md);
-
-/* MAGIC: perform f, a function that returns notification_t
- * and return from the ENCLOSING stf_status returning function if it fails.
- */
-/* macro that returns STF_STATUS on failure */
-#define RETURN_STF_FAILURE_STATUS(f) { \
-	stf_status res = (f); \
-	if (res != STF_OK) { \
-		return res; \
-	} \
-}
-
 struct ikev2_proposal;
 struct ikev2_proposals;
 
@@ -123,11 +76,11 @@ struct ikev2_proposals *get_v2_create_child_proposals(struct connection *c, cons
 
 bool ikev2_emit_sa_proposal(struct pbs_out *pbs,
 			    const struct ikev2_proposal *proposal,
-			    const chunk_t *local_spi);
+			    shunk_t local_spi);
 
 bool ikev2_emit_sa_proposals(struct pbs_out *outs,
 			     const struct ikev2_proposals *proposals,
-			     const chunk_t *local_spi);
+			     const shunk_t local_spi);
 
 const struct dh_desc *ikev2_proposals_first_dh(const struct ikev2_proposals *proposals,
 					       struct logger *logger);
@@ -152,13 +105,6 @@ bool ikev2_proposal_to_proto_info(const struct ikev2_proposal *proposal,
 bool ikev2_proposal_to_trans_attrs(const struct ikev2_proposal *chosen,
 				   struct trans_attrs *ta_out, struct logger *logger);
 
-struct ipsec_proto_info *ikev2_child_sa_proto_info(struct child_sa *child, lset_t policy);
-
-ipsec_spi_t ikev2_child_sa_spi(const struct spd_route *spd_route, lset_t policy,
-			       struct logger *logger);
-
-extern bool ikev2_decode_peer_id(struct msg_digest *md);
-
 extern void ikev2_log_parentSA(const struct state *st);
 
 extern bool ikev2_calculate_rsa_hash(struct ike_sa *ike,
@@ -177,36 +123,21 @@ extern bool ikev2_create_psk_auth(enum keyword_authby authby,
 				  const struct crypt_mac *idhash,
 				  chunk_t *additional_auth /* output */);
 
-extern stf_status ikev2_verify_rsa_hash(struct ike_sa *ike,
-					const struct crypt_mac *idhash,
-					pb_stream *sig_pbs,
-					const struct hash_desc *hash_algo);
+diag_t v2_authsig_and_log_using_RSA_pubkey(struct ike_sa *ike,
+					   const struct crypt_mac *idhash,
+					   shunk_t signature,
+					   const struct hash_desc *hash_algo);
 
-extern stf_status ikev2_verify_ecdsa_hash(struct ike_sa *ike,
-					const struct crypt_mac *idhash,
-					pb_stream *sig_pbs,
-					const struct hash_desc *hash_algo);
+diag_t v2_authsig_and_log_using_ECDSA_pubkey(struct ike_sa *ike,
+					     const struct crypt_mac *idhash,
+					     shunk_t signature,
+					     const struct hash_desc *hash_algo);
 
-extern bool ikev2_verify_psk_auth(enum keyword_authby authby,
-				  const struct ike_sa *ike,
-				  const struct crypt_mac *idhash,
-				  pb_stream *sig_pbs);
+extern void ikev2_derive_child_keys(struct ike_sa *ike, struct child_sa *child);
 
-extern void ikev2_derive_child_keys(struct child_sa *child);
+void schedule_v2_replace_event(struct state *st);
 
-stf_status ikev2_child_sa_respond(struct ike_sa *ike,
-				  struct child_sa *child,
-				  struct msg_digest *md,
-				  pb_stream *outpbs,
-				  enum isakmp_xchg_types isa_xchg);
-
-void v2_schedule_replace_event(struct state *st);
-
-bool emit_v2_child_configuration_payload(struct connection *c,
-					 struct child_sa *child,
-					 pb_stream *outpbs);
-
-bool ikev2_parse_cp_r_body(struct payload_digest *cp_pd, struct state *st);
+bool ikev2_parse_cp_r_body(struct payload_digest *cp_pd, struct child_sa *child);
 
 struct ikev2_payload_errors {
 	bool bad;
@@ -225,11 +156,11 @@ struct ikev2_expected_payloads {
 	v2_notification_t notification;
 };
 
-struct state_v2_microcode {
+struct v2_state_transition {
 	const char *const story;	/* state transition story (not state_story[]) */
 	const enum state_kind state;
 	const enum state_kind next_state;
-	const enum isakmp_xchg_types recv_type;
+	const enum isakmp_xchg_type recv_type;
 	enum message_role recv_role;
 	const lset_t flags;
 
@@ -265,30 +196,60 @@ void ikev2_copy_cookie_from_sa(const struct ikev2_proposal *accepted_ike_proposa
 				ike_spi_t *cookie);
 
 void ikev2_ike_sa_established(struct ike_sa *ike,
-			      const struct state_v2_microcode *svm,
+			      const struct v2_state_transition *svm,
 			      enum state_kind new_state);
 
-struct ikev2_ipseckey_dns;
-
-extern stf_status ikev2_process_child_sa_pl(struct ike_sa *ike, struct child_sa *child,
-					    struct msg_digest *md, bool expect_accepted);
-
-extern bool emit_v2KE(chunk_t *g, const struct dh_desc *group, pb_stream *outs);
+extern bool emit_v2KE(chunk_t g, const struct dh_desc *group, pb_stream *outs);
 
 extern void init_ikev2(void);
 
-extern void ikev2_record_newaddr(struct state *st, void *arg_ip);
-extern void ikev2_record_deladdr(struct state *st, void *arg_ip);
-extern void ikev2_addr_change(struct state *st);
-
 void jam_v2_stf_status(struct jambuf *buf, unsigned ret);
 
 void v2_event_sa_rekey(struct state *st);
+void v2_event_sa_reauth(struct state *st);
 void v2_event_sa_replace(struct state *st);
 
-/* used by parent and child to emit v2N_IPCOMP_SUPPORTED if appropriate */
-bool emit_v2N_compression(struct state *cst,
-			bool OK,
-			pb_stream *s);
+struct payload_summary ikev2_decode_payloads(struct logger *log,
+					     struct msg_digest *md,
+					     pb_stream *in_pbs,
+					     enum next_payload_types_ikev2 np);
+
+void v2_dispatch(struct ike_sa *ike, struct state *st,
+		 struct msg_digest *md,
+		 const struct v2_state_transition *transition);
+
+bool accept_v2_nonce(struct logger *logger, struct msg_digest *md,
+		     chunk_t *dest, const char *name);
+
+bool v2_accept_ke_for_proposal(struct ike_sa *ike,
+			       struct state *st,
+			       struct msg_digest *md,
+			       const struct dh_desc *accepted_dh,
+			       enum payload_security security);
+bool need_v2_configuration_payload(const struct connection *const pc,
+				   const lset_t st_nat_traversal);
+void ikev2_rekey_expire_predecessor(const struct child_sa *larval_sa, so_serial_t pred);
+
+struct crypt_mac v2_id_hash(struct ike_sa *ike, const char *why,
+			    const char *id_name, shunk_t id_payload,
+			    const char *key_name, PK11SymKey *key);
+bool id_ipseckey_allowed(struct ike_sa *ike, enum ikev2_auth_method atype);
+struct crypt_mac v2_hash_id_payload(const char *id_name, struct ike_sa *ike,
+				    const char *key_name, PK11SymKey *key);
+
+void IKE_SA_established(const struct ike_sa *ike);
+
+bool negotiate_hash_algo_from_notification(const struct pbs_in *payload_pbs,
+					   struct ike_sa *ike);
+
+
+/*
+ * See 2.21. Error Handling.  In particular the IKE_AUTH discussion.
+ */
+
+bool v2_notification_fatal(v2_notification_t n);
+stf_status stf_status_from_v2_notification(v2_notification_t n);
+
+bool already_has_larval_v2_child(struct ike_sa *ike, const struct connection *c);
 
 #endif
