@@ -42,15 +42,6 @@ struct finite_state v2_states[] = {
 		.kind = KIND,					\
 		.name = #KIND,					\
 		/* Not using #KIND + 6 because of clang's -Wstring-plus-int */ \
-		.short_name = &#KIND[6]/*STATE_*/,		\
-		.story = STORY,					\
-		.category = CAT,				\
-	}
-
-#define V2(KIND, STORY, CAT) [KIND - STATE_IKEv2_FLOOR] = {	\
-		.kind = KIND,					\
-		.name = #KIND,					\
-		/* Not using #KIND + 9 because of clang's -Wstring-plus-int */ \
 		.short_name = &#KIND[9]/*STATE_V2_*/,		\
 		.story = STORY,					\
 		.category = CAT,				\
@@ -59,11 +50,11 @@ struct finite_state v2_states[] = {
 	/*
 	 * IKEv2 IKE SA initiator, while the the SA_INIT packet is
 	 * being constructed, are in state.  Only once the packet has
-	 * been sent out does it transition to STATE_PARENT_I1 and
+	 * been sent out does it transition to STATE_V2_PARENT_I1 and
 	 * start being counted as half-open.
 	 */
 
-	S(STATE_PARENT_I0, "waiting for KE to finish", CAT_IGNORE),
+	S(STATE_V2_PARENT_I0, "waiting for KE to finish", CAT_IGNORE),
 
 	/*
 	 * Count I1 as half-open too because with ondemand, a
@@ -71,16 +62,16 @@ struct finite_state v2_states[] = {
 	 * IKE SA.
 	 */
 
-	S(STATE_PARENT_I1, "sent IKE_SA_INIT request", CAT_HALF_OPEN_IKE_SA),
-	S(STATE_PARENT_R0, "processing IKE_SA_INIT request", CAT_HALF_OPEN_IKE_SA),
-	S(STATE_PARENT_R1, "sent IKE_SA_INIT reply", CAT_HALF_OPEN_IKE_SA),
+	S(STATE_V2_PARENT_I1, "sent IKE_SA_INIT request", CAT_HALF_OPEN_IKE_SA),
+	S(STATE_V2_PARENT_R0, "processing IKE_SA_INIT request", CAT_HALF_OPEN_IKE_SA),
+	S(STATE_V2_PARENT_R1, "sent IKE_SA_INIT reply", CAT_HALF_OPEN_IKE_SA),
 
 	/*
 	 * All IKEv1 MAIN modes except the first (half-open) and last
 	 * ones are not authenticated.
 	 */
 
-	S(STATE_PARENT_I2, "sent IKE_AUTH request", CAT_OPEN_IKE_SA),
+	S(STATE_V2_PARENT_I2, "sent IKE_AUTH request", CAT_OPEN_IKE_SA),
 
 	/* IKE exchange can also create a child */
 
@@ -106,13 +97,13 @@ struct finite_state v2_states[] = {
 	 * IKEv2 established states.
 	 */
 
-	V2(STATE_V2_ESTABLISHED_IKE_SA, "established IKE SA", CAT_ESTABLISHED_IKE_SA),
+	S(STATE_V2_ESTABLISHED_IKE_SA, "established IKE SA", CAT_ESTABLISHED_IKE_SA),
 	/* this message is used for both initial exchanges and rekeys */
-	V2(STATE_V2_ESTABLISHED_CHILD_SA, "IPsec SA established", CAT_ESTABLISHED_CHILD_SA),
+	S(STATE_V2_ESTABLISHED_CHILD_SA, "established Child SA", CAT_ESTABLISHED_CHILD_SA),
 
 	/* ??? better story needed for these */
-	S(STATE_IKESA_DEL, "STATE_IKESA_DEL", CAT_ESTABLISHED_IKE_SA),
-	S(STATE_CHILDSA_DEL, "STATE_CHILDSA_DEL", CAT_INFORMATIONAL),
+	S(STATE_V2_IKE_SA_DELETE, "STATE_IKESA_DEL", CAT_ESTABLISHED_IKE_SA),
+	S(STATE_V2_CHILD_SA_DELETE, "STATE_CHILDSA_DEL", CAT_INFORMATIONAL),
 #undef S
 };
 
@@ -181,8 +172,8 @@ struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 	}
 
 	if (payloads->notification != v2N_NOTHING_WRONG) {
-		enum v2_pbs v2_pbs = v2_notification_to_v2_pbs(payloads->notification);
-		if (md->pbs[v2_pbs] == NULL) {
+		enum v2_pd v2_pd = v2_pd_from_notification(payloads->notification);
+		if (md->pd[v2_pd] == NULL) {
 			errors.bad = true;
 			errors.notification = payloads->notification;
 		}
@@ -191,7 +182,7 @@ struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 	return errors;
 }
 
-const struct state_v2_microcode *find_v2_state_transition(struct logger *logger,
+const struct v2_state_transition *find_v2_state_transition(struct logger *logger,
 							  const struct finite_state *state,
 							  struct msg_digest *md)
 {
@@ -199,7 +190,7 @@ const struct state_v2_microcode *find_v2_state_transition(struct logger *logger,
 	struct ikev2_payload_errors message_payload_status = { .bad = false };
 	struct ikev2_payload_errors encrypted_payload_status = { .bad = false };
 	for (unsigned i = 0; i < state->nr_transitions; i++) {
-		const struct state_v2_microcode *transition = &state->v2_transitions[i];
+		const struct v2_state_transition *transition = &state->v2_transitions[i];
 		dbg("  trying %s", transition->story);
 		/* message type? */
 		if (transition->recv_type != md->hdr.isa_xchg) {
@@ -289,7 +280,7 @@ void log_v2_payload_errors(struct logger *logger, struct msg_digest *md,
 	}
 
 	LLOG_JAMBUF(RC_LOG_SERIOUS | log_stream, logger, buf) {
-		const enum isakmp_xchg_types ix = md->hdr.isa_xchg;
+		const enum isakmp_xchg_type ix = md->hdr.isa_xchg;
 		jam(buf, "dropping unexpected ");
 		jam_enum_short(buf, &ikev2_exchange_names, ix);
 		jam(buf, " message");
@@ -337,7 +328,7 @@ void log_v2_payload_errors(struct logger *logger, struct msg_digest *md,
 	}
 }
 
-void jam_v2_transition(struct jambuf *buf, const struct state_v2_microcode *transition)
+void jam_v2_transition(struct jambuf *buf, const struct v2_state_transition *transition)
 {
 	if (transition == NULL) {
 		jam(buf, "NULL");
