@@ -50,20 +50,22 @@
 #include "timer.h"
 #include "ikev2.h"
 #include "ip_address.h"
-#include "hostpair.h"
+#include "host_pair.h"
+#include "ikev2_create_child_sa.h"		/* for initiate_v2_CREATE_CHILD_SA_create_child() */
 
 /*
  * queue an IPsec SA negotiation pending completion of a
  * suitable phase 1 (IKE SA)
  */
-void add_pending(struct fd *whack_sock,
-		 struct ike_sa *ike,
-		 struct connection *c,
-		 lset_t policy,
-		 unsigned long try,
-		 so_serial_t replacing,
-		 const chunk_t sec_label,
-		 bool part_of_initiate)
+
+static void add_pending(struct fd *whack_sock,
+			struct ike_sa *ike,
+			struct connection *c,
+			lset_t policy,
+			unsigned long try,
+			so_serial_t replacing,
+			const shunk_t sec_label,
+			bool part_of_initiate)
 {
 	/* look for duplicate pending IPsec SA's, skip add operation */
 	struct pending **pp = host_pair_first_pending(c);
@@ -84,7 +86,7 @@ void add_pending(struct fd *whack_sock,
 	}
 
 	struct pending *p = alloc_thing(struct pending, "struct pending");
-	p->whack_sock = dup_any(whack_sock); /*on heap*/
+	p->whack_sock = fd_dup(whack_sock, HERE); /*on heap*/
 	p->ike = ike;
 	p->connection = c;
 	p->policy = policy;
@@ -101,16 +103,63 @@ void add_pending(struct fd *whack_sock,
 	enum stream only = part_of_initiate ? (DBGP(DBG_BASE) ? DEBUG_STREAM : NO_STREAM) : ALL_STREAMS;
 	if (only != NO_STREAM) {
 		address_buf b;
-		connection_buf cibb;
-		struct connection *cb = ike->sa.st_connection;
+		state_buf sab;
 		log_pending(only | RC_COMMENT, p,
-			    "queuing pending IPsec SA negotiating with %s IKE SA #%lu "PRI_CONNECTION"",
-			    ipstr(&c->spd.that.host_addr, &b),
-			    ike->sa.st_serialno, pri_connection(cb, &cibb));
+			    "queue %s; waiting on %s "PRI_STATE" negotiating with %s",
+			    /* "Child SA" or "IPsec SA" */
+			    enum_enum_name(&sa_type_names, p->connection->ike_version, IPSEC_SA),
+			    /* "IKE SA" or "ISAKMP SA" */
+			    enum_enum_name(&sa_type_names, p->connection->ike_version, IKE_SA),
+			    pri_state(&ike->sa, &sab),
+			    ipstr(&c->spd.that.host_addr, &b));
 	}
 	host_pair_enqueue_pending(c, p);
 }
 
+void add_v1_pending(struct fd *whackfd,
+		    struct ike_sa *ike,
+		    struct connection *c,
+		    lset_t policy,
+		    unsigned long try,
+		    so_serial_t replacing,
+		    const shunk_t sec_label,
+		    bool part_of_initiate)
+{
+	passert(ike->sa.st_ike_version == IKEv1);
+	add_pending(whackfd, ike, c, policy, try, replacing, sec_label, part_of_initiate);
+}
+
+void add_v2_pending(struct fd *whackfd,
+		    struct ike_sa *ike,
+		    struct connection *c,
+		    lset_t policy,
+		    unsigned long try,
+		    so_serial_t replacing,
+		    const shunk_t sec_label,
+		    bool part_of_initiate)
+{
+	passert(ike->sa.st_ike_version == IKEv2);
+	if (c->config->sec_label.len > 0 || sec_label.len > 0) {
+		/*
+		 * Convert the template connection into a connection
+		 * instance that contains the sec_label, and toss that
+		 * onto the pending queue.
+		 */
+		if (!pexpect(c->kind == CK_TEMPLATE) ||
+		    !pexpect(c->config->sec_label.len > 0) ||
+		    !pexpect(sec_label.len > 0)) {
+			return;
+		}
+		ip_address remote_address = endpoint_address(ike->sa.st_remote_endpoint);
+		struct connection *d = instantiate(c, &remote_address, /*peer_id*/NULL, sec_label);
+		connection_buf db;
+		dbg("generating and then tossing child connection "PRI_CONNECTION" with sec_label="PRI_SHUNK" into the pending queue",
+		    pri_connection(d, &db), pri_shunk(sec_label));
+		c = d;
+	}
+	add_pending(whackfd, ike, c, policy, try, replacing, sec_label, part_of_initiate);
+}
+
 /*
  * Release all the whacks awaiting the completion of this state.  This
  * is accomplished by closing all the whack socket file descriptors.
@@ -183,10 +232,8 @@ void release_pending_whacks(struct state *st, err_t story)
 				log_pending(WHACK_STREAM|RC_COMMENT, p,
 					    "%s for IKE SA, but releasing whack for pending %s",
 					    story,
-					    /* IPsec SA or CHILD SA */
-					    enum_enum_name(&sa_type_names,
-							   p->connection->ike_version,
-							   IPSEC_SA));
+					    /* "IPsec SA" or "CHILD SA" */
+					    enum_enum_name(&sa_type_names, p->connection->ike_version, IPSEC_SA));
 			}
 			close_any(&p->whack_sock);/*on-heap*/
 		}
@@ -249,11 +296,9 @@ static void delete_pending(struct pending **pp)
  *     In IKEv2 it called when AUTH is complete, child is established.
  *     Established child get removed not unpend.
  */
+
 void unpend(struct ike_sa *ike, struct connection *cc)
 {
-	struct pending **pp, *p;
-	char *what ="unqueuing";
-
 	if (cc == NULL) {
 		dbg("unpending state #%lu", ike->sa.st_serialno);
 	} else {
@@ -262,16 +307,14 @@ void unpend(struct ike_sa *ike, struct connection *cc)
 		    ike->sa.st_serialno, pri_connection(cc, &cib));
 	}
 
-	for (pp = host_pair_first_pending(ike->sa.st_connection);
-	     (p = *pp) != NULL; )
-	{
+	for (struct pending *p, **pp = host_pair_first_pending(ike->sa.st_connection);
+	     (p = *pp) != NULL; ) {
 		if (p->ike == ike) {
 			p->pend_time = mononow();
+			char *what ="unqueuing";
 			switch (ike->sa.st_ike_version) {
 			case IKEv2:
-				if (cc != p->connection) {
-					ikev2_initiate_child_sa(p);
-				} else {
+				if (cc == p->connection) {
 					/*
 					 * IKEv2 AUTH negotiation
 					 * include child.  nothing to
@@ -279,13 +322,18 @@ void unpend(struct ike_sa *ike, struct connection *cc)
 					 * delete it
 					 */
 					what = "delete from";
+				} else if (!already_has_larval_v2_child(ike, p->connection)) {
+					submit_v2_CREATE_CHILD_SA_new_child(ike, p->connection,
+									    p->policy, p->try,
+									    p->whack_sock);
 				}
 				break;
 			case IKEv1:
 #ifdef USE_IKEv1
 				quick_outI1(p->whack_sock, &ike->sa, p->connection,
 					    p->policy,
-					    p->try, p->replacing, empty_chunk);
+					    p->try, p->replacing,
+					    null_shunk);
 #endif
 				break;
 			default:
@@ -320,7 +368,7 @@ struct connection *first_pending(const struct ike_sa *ike,
 	{
 		if (p->ike == ike) {
 			close_any(p_whack_sock); /*on-heap*/
-			*p_whack_sock = dup_any(p->whack_sock); /*on-heap*/
+			*p_whack_sock = fd_dup(p->whack_sock, HERE); /*on-heap*/
 			*policy = p->policy;
 			return p->connection;
 		}
@@ -350,12 +398,12 @@ bool pending_check_timeout(const struct connection *c)
 				connection_buf cib;
 				dbg("connection "PRI_CONNECTION" stuck, restarting",
 				    pri_connection(c, &cib));
-				return TRUE;
+				return true;
 			}
 		}
 		pp = &p->next;
 	}
-	return FALSE;
+	return false;
 }
 
 /* a IKE SA negotiation has been replaced; update any pending */
@@ -417,9 +465,9 @@ void flush_pending_by_connection(const struct connection *c)
 	}
 }
 
-void show_pending_phase2(struct show *s,
-			 const struct connection *c,
-			 const struct ike_sa *ike)
+void show_pending_child_details(struct show *s,
+				const struct connection *c,
+				const struct ike_sa *ike)
 {
 	struct pending **pp, *p;
 
@@ -450,11 +498,11 @@ bool connection_is_pending(const struct connection *c)
 
 	pp = host_pair_first_pending(c);
 	if (pp == NULL)
-		return FALSE;
+		return false;
 
 	for (p = *pp; p != NULL; p = p->next)
 		if (p->connection == c)
-			return TRUE; /* in use, so we're done */
+			return true; /* in use, so we're done */
 
-	return FALSE;
+	return false;
 }
