@@ -31,9 +31,11 @@
 #endif
 #include "passert.h"
 
+#include "jambuf.h"
 #include "constants.h"
 #include "enum_names.h"
 #include "defs.h"
+#include "kernel.h"
 
 /*
  * To obsolete or convert to runtime options:
@@ -105,49 +107,8 @@ enum_names sd_action_names = {
 	NULL
 };
 
-/* Timer events */
-static const char *const timer_event_name[] = {
-#define E(EVENT) [EVENT] = #EVENT
-
-	E(EVENT_NULL),
-
-	E(EVENT_SO_DISCARD),
-	E(EVENT_RETRANSMIT),
-
-	E(EVENT_SA_REKEY),
-	E(EVENT_SA_REPLACE),
-	E(EVENT_SA_EXPIRE),
-
-	E(EVENT_v1_SEND_XAUTH),
-	E(EVENT_v1_SA_REPLACE_IF_USED),
-	E(EVENT_DPD),
-	E(EVENT_DPD_TIMEOUT),
-	E(EVENT_CRYPTO_TIMEOUT),
-	E(EVENT_PAM_TIMEOUT),
-
-	E(EVENT_v2_LIVENESS),
-	E(EVENT_v2_RELEASE_WHACK),
-	E(EVENT_v2_INITIATE_CHILD),
-	E(EVENT_v2_ADDR_CHANGE),
-	E(EVENT_v2_REDIRECT),
-	E(EVENT_RETAIN),
-
-#undef E
-};
-
-enum_names timer_event_names = {
-	EVENT_NULL, EVENT_RETAIN,
-	ARRAY_REF(timer_event_name),
-	NULL, /* prefix */
-	NULL
-};
-
-/*
- * natt_bit_names is dual purpose:
- * - for bitnamesof(natt_bit_names, lset_t of enum natt_method)
- * - for enum_name(&natt_method_names, enum natt_method)
- */
-const char *const natt_bit_names[] = {
+/* NAT methods */
+static const char *const natt_method_name[] = {
 	"none",
 	"draft-ietf-ipsec-nat-t-ike-02/03",
 	"draft-ietf-ipsec-nat-t-ike-05",
@@ -155,12 +116,11 @@ const char *const natt_bit_names[] = {
 
 	"I am behind NAT",
 	"peer behind NAT",
-	NULL	/* end for bitnamesof() */
 };
 
 enum_names natt_method_names = {
 	NAT_TRAVERSAL_METHOD_none, NATED_PEER,
-	ARRAY_REF(natt_bit_names)-1,
+	ARRAY_REF(natt_method_name),
 	NULL, /* prefix */
 	NULL
 };
@@ -190,7 +150,7 @@ static const char *const stf_status_strings[] = {
 	A(STF_SUSPEND),
 	A(STF_OK),
 	A(STF_INTERNAL_ERROR),
-	A(STF_V2_DELETE_EXCHANGE_INITIATOR_IKE_SA),
+	A(STF_V2_DELETE_IKE_AUTH_INITIATOR),
 	A(STF_FATAL),
 	A(STF_FAIL),
 #undef A
@@ -205,70 +165,83 @@ enum_names stf_status_names = {
 
 /*
  * Names for sa_policy_bits.
- * Note: we drop the POLICY_ prefix so that logs are more concise.
  */
-const char *const sa_policy_bit_names[] = {
-	"PSK",
-	"RSASIG",
-	"ECDSA",
-	"AUTH_NEVER",
-	"AUTHNULL",
-	"ENCRYPT",
-	"AUTHENTICATE",
-	"COMPRESS",
-	"TUNNEL",
-	"PFS",
-	"DECAP_DSCP",
-	"NOPMTUDISC",
-	"MSDH_DOWNGRADE",
-	"ALLOW_NO_SAN",
-	"DNS_MATCH_ID",
-	"SHA2_TRUNCBUG",
-	"SHUNT0",
-	"SHUNT1",
-	"FAIL0",
-	"FAIL1",
-	"NEGO_PASS",
-	"DONT_REKEY",
-	"REAUTH",
-	"OPPORTUNISTIC",
-	"GROUP",
-	"GROUTED",
-	"GROUPINSTANCE",
-	"UP",
-	"XAUTH",
-	"MODECFG_PULL",
-	"AGGRESSIVE",
-	"OVERLAPIP",
-	"IKEV1_ALLOW",
-	"IKEV2_ALLOW",
-	"IKEV2_ALLOW_NARROWING",
-	"IKEV2_PAM_AUTHORIZE",
-	"SEND_REDIRECT_ALWAYS",
-	"SEND_REDIRECT_NEVER",
-	"ACCEPT_REDIRECT_YES",
-	"IKE_FRAG_ALLOW",
-	"IKE_FRAG_FORCE",
-	"NO_IKEPAD",
-	"MOBIKE",
-	"PPK_ALLOW",
-	"PPK_INSIST",
-	"ESN_NO",
-	"ESN_YES",
-	"INTERMEDIATE",
-	"IGNORE_PEER_DNS",
-	"RSASIG_v1_5",
-	NULL	/* end for bitnamesof() */
+static const char *const sa_policy_bit_name[] = {
+#define P(N) [N##_IX] = #N
+	P(POLICY_PSK),
+	P(POLICY_RSASIG),
+	P(POLICY_ECDSA),
+	P(POLICY_AUTH_NEVER),
+	P(POLICY_AUTH_NULL),
+	P(POLICY_ENCRYPT),
+	P(POLICY_AUTHENTICATE),
+	P(POLICY_COMPRESS),
+	P(POLICY_TUNNEL),
+	P(POLICY_PFS),
+	P(POLICY_DECAP_DSCP),
+	P(POLICY_NOPMTUDISC),
+	P(POLICY_MSDH_DOWNGRADE),
+	P(POLICY_ALLOW_NO_SAN),
+	P(POLICY_DNS_MATCH_ID),
+	P(POLICY_SHA2_TRUNCBUG),
+	P(POLICY_SHUNT0),
+	P(POLICY_SHUNT1),
+	P(POLICY_FAIL0),
+	P(POLICY_FAIL1),
+	P(POLICY_NEGO_PASS),
+	P(POLICY_DONT_REKEY),
+	P(POLICY_REAUTH),
+	P(POLICY_OPPORTUNISTIC),
+	P(POLICY_GROUP),
+	P(POLICY_GROUTED),
+	P(POLICY_GROUPINSTANCE),
+	P(POLICY_UP),
+	P(POLICY_XAUTH),
+	P(POLICY_MODECFG_PULL),
+	P(POLICY_AGGRESSIVE),
+	P(POLICY_OVERLAPIP),
+	P(POLICY_IKEV2_ALLOW_NARROWING),
+	P(POLICY_IKEV2_PAM_AUTHORIZE),
+	P(POLICY_SEND_REDIRECT_ALWAYS),
+	P(POLICY_SEND_REDIRECT_NEVER),
+	P(POLICY_ACCEPT_REDIRECT_YES),
+	P(POLICY_IKE_FRAG_ALLOW),
+	P(POLICY_IKE_FRAG_FORCE),
+	P(POLICY_NO_IKEPAD),
+	P(POLICY_MOBIKE),
+	P(POLICY_PPK_ALLOW),
+	P(POLICY_PPK_INSIST),
+	P(POLICY_ESN_NO),
+	P(POLICY_ESN_YES),
+	P(POLICY_INTERMEDIATE),
+	P(POLICY_IGNORE_PEER_DNS),
+	P(POLICY_RSASIG_v1_5),
+#undef P
 };
 
+static const enum_names sa_policy_bit_names = {
+	0, POLICY_IX_LAST,
+	ARRAY_REF(sa_policy_bit_name),
+	"POLICY_", /* prefix */
+	NULL
+};
+
+
 /*
  * Names for RFC 7427 IKEv2 AUTH signature hash algo sighash_policy_bits
  */
-const char *const sighash_policy_bit_names[] = {
+static const char *const sighash_policy_bit_name[] = {
 	"SHA2_256",
 	"SHA2_384",
 	"SHA2_512",
-	NULL	/* end for bitnamesof() */
+};
+
+const struct enum_names sighash_policy_bit_names = {
+	POL_SIGHASH_SHA2_256_IX,
+	POL_SIGHASH_SHA2_512_IX,
+	ARRAY_REF(sighash_policy_bit_name),
+	NULL, /* prefix */
+	NULL, /* next */
 };
 
 static const char *const keyword_authby_name[] = {
@@ -346,7 +319,7 @@ enum_names v1_sa_type_names = {
 
 static const char *const v2_sa_type_name[] = {
 	[IKE_SA] = "IKE SA",
-	[IPSEC_SA] = "CHILD SA"
+	[IPSEC_SA] = "Child SA"
 };
 
 enum_names v2_sa_type_names = {
@@ -356,7 +329,7 @@ enum_names v2_sa_type_names = {
 	NULL,
 };
 
-static enum_names *sa_type_name[] = {
+static const enum_names *sa_type_name[] = {
 	[IKEv1 - IKEv1] = &v1_sa_type_names,
 	[IKEv2 - IKEv1] = &v2_sa_type_names,
 };
@@ -366,6 +339,28 @@ enum_enum_names sa_type_names = {
 	ARRAY_REF(sa_type_name),
 };
 
+/* enum kernel_policy_op_names */
+
+static const char *kernel_policy_op_name[] = {
+	[0] = "KP_INVALID",
+#define S(E) [E] = #E
+	S(KP_ADD_OUTBOUND),
+	S(KP_REPLACE_OUTBOUND),
+	S(KP_DELETE_OUTBOUND),
+	S(KP_ADD_INBOUND),
+	S(KP_REPLACE_INBOUND),
+	S(KP_DELETE_INBOUND),
+#undef S
+};
+
+enum_names kernel_policy_op_names = {
+	0, elemsof(kernel_policy_op_name)-1,
+	ARRAY_REF(kernel_policy_op_name),
+	.en_prefix = "KP_",
+};
+
+/* */
+
 static const char *const perspective_name[] = {
 	[NO_PERSPECTIVE] = "NO_PERSPECTIVE",
 	[LOCAL_PERSPECTIVE] = "LOCAL_PERSPECTIVE",
@@ -382,33 +377,42 @@ enum_names perspective_names = {
 /* print a policy: like bitnamesof, but it also does the non-bitfields.
  * Suppress the shunt and fail fields if 0.
  */
-const char *prettypolicy(lset_t policy)
-{
-	char pbitnamesbuf[200];
-	const char *bn = bitnamesofb(sa_policy_bit_names,
-				     policy &
-				     ~(POLICY_SHUNT_MASK | POLICY_FAIL_MASK),
-				     pbitnamesbuf, sizeof(pbitnamesbuf));
-	static char buf[512]; /* NOT RE-ENTRANT!  I hope that it is big enough! */
-	lset_t shunt = (policy & POLICY_SHUNT_MASK) >> POLICY_SHUNT_SHIFT;
-	lset_t fail = (policy & POLICY_FAIL_MASK) >> POLICY_FAIL_SHIFT;
 
-	if (bn != pbitnamesbuf)
-		pbitnamesbuf[0] = '\0';
-	snprintf(buf, sizeof(buf), "%s%s%s%s%s",
-		 pbitnamesbuf,
-		 shunt == POLICY_SHUNT_TRAP >> POLICY_SHUNT_SHIFT ?  "" : "+",
-		 shunt ==  POLICY_SHUNT_TRAP >> POLICY_SHUNT_SHIFT ? "" : policy_shunt_names[shunt],
-		 fail == POLICY_FAIL_NONE >> POLICY_FAIL_SHIFT ? "" : "+failure",
-		 fail == POLICY_FAIL_NONE >> POLICY_FAIL_SHIFT ? "" : policy_fail_names[fail]);
-	return buf;
+size_t jam_policy(struct jambuf *buf, lset_t policy)
+{
+	size_t s = 0;
+
+	lset_t other = policy & ~(POLICY_SHUNT_MASK | POLICY_FAIL_MASK);
+	const char *sep = "";
+	if (other != LEMPTY) {
+		s += jam_lset_short(buf, &sa_policy_bit_names, "+", other);
+		sep = "+";
+	}
+
+	lset_t shunt = (policy & POLICY_SHUNT_MASK);
+	if (shunt != POLICY_SHUNT_TRAP) {
+		s += jam(buf, "%s%s", sep, policy_shunt_names[shunt >> POLICY_SHUNT_SHIFT]);
+		sep = "+";
+	}
+	lset_t fail = (policy & POLICY_FAIL_MASK);
+	if (fail != POLICY_FAIL_NONE) {
+		s += jam(buf, "%sfailure%s", sep, policy_fail_names[fail >> POLICY_FAIL_SHIFT]);
+		sep = "+";
+	}
+	return s;
+}
+
+const char *str_policy(lset_t policy, policy_buf *dst)
+{
+	struct jambuf buf = ARRAY_AS_JAMBUF(dst->buf);
+	jam_policy(&buf, policy);
+	return dst->buf;
 }
 
 static const enum_names *pluto_enum_names_checklist[] = {
 	&kern_interface_names,
 	&dpd_action_names,
 	&sd_action_names,
-	&timer_event_names,
 	&natt_method_names,
 	&routing_story,
 	&stf_status_names,
@@ -418,6 +422,8 @@ static const enum_names *pluto_enum_names_checklist[] = {
 	&v1_sa_type_names,
 	&v2_sa_type_names,
 	&perspective_names,
+	&sa_policy_bit_names,
+	&kernel_policy_op_names,
 };
 
 void init_pluto_constants(void) {
