@@ -4,6 +4,7 @@
  * Copyright (C) 2017 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2019 Andrew Cagney
  * Copyright (C) 2019 D. Hugh Redelmeier <hugh@mimosa.com>
+ * Copyright (C) 2021 Paul Wouters <paul.wouters@aiven.io>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -46,6 +47,8 @@ unsigned long pstats_ikev1_fail;
 unsigned long pstats_ikev2_fail;
 unsigned long pstats_ikev1_completed;
 unsigned long pstats_ikev2_completed;
+unsigned long pstats_ikev2_redirect_failed;
+unsigned long pstats_ikev2_redirect_completed;
 unsigned long pstats_ikev1_encr[OAKLEY_ENCR_PSTATS_ROOF];
 unsigned long pstats_ikev2_encr[IKEv2_ENCR_PSTATS_ROOF];
 unsigned long pstats_ikev1_integ[OAKLEY_HASH_PSTATS_ROOF];
@@ -140,7 +143,9 @@ static const char *pstats_sa_reasons[DELETE_REASON_ROOF] = {
 	[REASON_EXCHANGE_TIMEOUT] = "exchange-timeout",
 	[REASON_CRYPTO_TIMEOUT] = "crypto-timeout",
 	[REASON_TOO_MANY_RETRANSMITS] = "too-many-retransmits",
+	[REASON_SUPERSEDED_BY_NEW_SA] = "superseeded-by-new-sa",
 	[REASON_AUTH_FAILED] = "auth-failed",
+	[REASON_TRAFFIC_SELECTORS_FAILED] = "ts-unacceptable",
 	[REASON_CRYPTO_FAILED] = "crypto-failed",
 };
 
@@ -184,7 +189,9 @@ void pstat_sa_deleted(struct state *st)
 	 * XXX: the above should make this completely redundant.
 	 */
 	if (IS_IKE_SA(st)) {
-		bool fail = !IS_IKE_SA_ESTABLISHED(st) && st->st_state->kind != STATE_IKESA_DEL;
+		bool fail = (!IS_IKE_SA_ESTABLISHED(st) &&
+			     !IS_V1_ISAKMP_SA_ESTABLISHED(st) &&
+			     st->st_state->kind != STATE_V2_IKE_SA_DELETE);
 		if (fail) {
 			if (st->st_ike_version == IKEv2)
 				pstats_ikev2_fail++;
@@ -283,7 +290,7 @@ static void pstat_child_sa_established(struct state *st)
 		pstatsv(ipsec_integ, (st->st_ike_version == IKEv2),
 			st->st_ah.attrs.transattrs.ta_integ->common.id[IKEv1_ESP_ID],
 			st->st_ah.attrs.transattrs.ta_integ->common.id[IKEv2_ALG_ID]);
-		pstats_sa(FALSE, FALSE, esn);
+		pstats_sa(FALSE, false, esn);
 	}
 	if (st->st_ipcomp.present) {
 		pstats_ipsec_ipcomp++;
@@ -320,7 +327,7 @@ static void whack_pluto_stat(struct show *s, const struct pluto_stat *stat)
 {
 	unsigned long other = stat->count[stat->roof - stat->floor];
 	for (unsigned long e = stat->floor; e < stat->roof; e++) {
-		const char *nm = enum_short_name(stat->names, e);
+		const char *nm = enum_name_short(stat->names, e);
 		unsigned long count = stat->count[e - stat->floor];
 		/* not logging "UNUSED" */
 		if (nm != NULL && strstr(nm, "UNUSED") == NULL) {
@@ -350,7 +357,7 @@ static void enum_stats(struct show *s, enum_names *names, unsigned long start,
 		 * XXX: the bug is that the enum table contains names
 		 * that include UNUSED.  Skip them.
 		 */
-		const char *name = enum_short_name(names, e);
+		const char *name = enum_name_short(names, e);
 		if (name != NULL && strstr(name, "UNUSED") == NULL) {
 			show_raw(s, "total.%s.%s=%lu",
 				 what, name, count[e]);
@@ -393,6 +400,8 @@ void show_pluto_stats(struct show *s)
 	show_raw(s, "total.ike.ikev2.established=%lu", pstats_ikev2_sa);
 	show_raw(s, "total.ike.ikev2.failed=%lu", pstats_ikev2_fail);
 	show_raw(s, "total.ike.ikev2.completed=%lu", pstats_ikev2_completed);
+	show_raw(s, "total.ike.ikev2.redirect.completed=%lu", pstats_ikev2_redirect_completed);
+	show_raw(s, "total.ike.ikev2.redirect.failed=%lu", pstats_ikev2_redirect_failed);
 	show_raw(s, "total.ike.ikev1.established=%lu", pstats_ikev1_sa);
 	show_raw(s, "total.ike.ikev1.failed=%lu", pstats_ikev1_fail);
 	show_raw(s, "total.ike.ikev1.completed=%lu", pstats_ikev1_completed);
@@ -482,6 +491,7 @@ void clear_pluto_stats(void)
 	pstats_ipsec_sa = pstats_ikev1_sa = pstats_ikev2_sa = 0;
 	pstats_ikev1_fail = pstats_ikev2_fail = 0;
 	pstats_ikev1_completed = pstats_ikev2_completed = 0;
+	pstats_ikev2_redirect_failed = pstats_ikev2_redirect_completed=0;
 
 	memset(pstats_sa_started, 0, sizeof pstats_sa_started);
 	memset(pstats_sa_finished, 0, sizeof pstats_sa_finished);
