@@ -35,44 +35,45 @@ struct connection;
 struct msg_digest;
 struct certs;
 struct ike_sa;
+struct cert;
 
-extern bool v2_decode_certs(struct ike_sa *ike, struct msg_digest *md);
+bool v1_decode_certs(struct msg_digest *md);
+bool v1_verify_certs(struct msg_digest *md);
 
-extern lsw_cert_ret v1_process_certs(struct msg_digest *md);
+diag_t match_end_cert_id(const struct certs *certs, struct id *peer_id /*ID_FROMCERT => updated*/);
 
-bool match_certs_id(const struct certs *certs, struct id *peer_id /*ID_FROMCERT => updated*/);
-
-extern void ikev1_decode_cr(struct msg_digest *md);
-extern void ikev2_decode_cr(struct msg_digest *md);
+extern void ikev1_decode_cr(struct msg_digest *md, struct logger *logger);
+extern void ikev2_decode_cr(struct msg_digest *md, struct logger *logger);
 
 extern generalName_t *collect_rw_ca_candidates(struct msg_digest *md);
 
 extern bool ikev1_build_and_ship_CR(enum ike_cert_type type,
-				    chunk_t ca, pb_stream *outs,
-				    enum next_payload_types_ikev1 np);
+				    chunk_t ca, pb_stream *outs);
 
 extern bool ikev2_build_and_ship_CR(enum ike_cert_type type,
 				    chunk_t ca, pb_stream *outs);
 
 extern void load_authcerts(const char *type, const char *path,
-			   u_char auth_flags);
+			   uint8_t auth_flags);
 
 extern bool match_requested_ca(const generalName_t *requested_ca,
 			       chunk_t our_ca, int *our_pathlen);
 
-extern bool ikev1_ship_CERT(uint8_t type, chunk_t cert, pb_stream *outs,
-							 uint8_t np);
+extern bool ikev1_ship_CERT(enum ike_cert_type type, shunk_t cert, pb_stream *outs);
 extern int get_auth_chain(chunk_t *out_chain, int chain_max,
-					      CERTCertificate *end_cert,
-					      bool full_chain);
+			  const struct cert *end_cert, bool full_chain);
 extern void free_auth_chain(chunk_t *chain, int chain_len);
-extern bool ikev2_send_cert_decision(const struct state *st);
+extern bool ikev2_send_cert_decision(const struct ike_sa *ike);
 extern stf_status ikev2_send_certreq(struct state *st, struct msg_digest *md,
 				     pb_stream *outpbs);
 
-stf_status ikev2_send_cert(const struct state *st, pb_stream *outpbs);
+stf_status ikev2_send_cert(const struct connection *c, pb_stream *outpbs);
 
 bool ikev2_send_certreq_INIT_decision(const struct state *st,
-				      enum original_role role);
+				      enum sa_role role);
+
+#if defined(LIBCURL) || defined(LIBLDAP)
+bool find_crl_fetch_dn(chunk_t *issuer_dn, struct connection *c);
+#endif
 
 #endif /* _PLUTO_X509_H */
