@@ -2,6 +2,6 @@
 ipsec _stackmanager start
 export PLUTO_CRYPTO_HELPER_DELAY=10
 ipsec pluto --config /etc/ipsec.conf --leak-detective
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add multi
 echo "initdone"
