@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep --x509
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec whack --impair suppress-retransmits
 ipsec auto --add road-east-x509-ipv4
 echo "initdone"
