@@ -3,7 +3,7 @@ ipsec auto --up north-east
 echo "3. north connection add+up done"
 sleep 1
 # should be connected to west!
-ping -n -c 4 192.0.2.254
+../../guestbin/ping-once.sh --up 192.0.2.254
 ipsec whack --trafficstatus
 ipsec auto --delete north-east
 echo "3. north connection delete done"
