@@ -4,7 +4,7 @@ cp east-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.3.0/24 tcp 22 0"  >> /etc/ipsec.d/policies/clear-or-private
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 # give OE policies time to load
-sleep 5
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
 echo "initdone"
