@@ -189,15 +189,16 @@ def json_result(logger, args, result):
             jsonutil.dump(json_result, output)
             output.write("\n")
 
-    # accumulate the results.
+    # accumulate results.json
     JSON_RESULTS.append(json_result)
 
-    # accumulate the summary
-    _add(JSON_SUMMARY, "totals", result.test.kind, result.test.status, str(result.resolution))
+    # accumulate summary.json
+    _add(JSON_SUMMARY, "totals", result.test.kind, result.test.status, str(result))
     for issue in result.issues:
         for domain in result.issues[issue]:
             # count the number of times it occurred
-            _add(JSON_SUMMARY, "errors", issue)
+            _add(JSON_SUMMARY, "totals", result.test.kind, result.test.status, "errors", issue)
+
     # extend the times
     _update_time(min, "start_time", json_result)
     _update_time(max, "stop_time", json_result)
